/***************************************************************************
 *            Anim.cpp
 *
 *  2005  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Anim.hpp"

using namespace std;

Anim::Anim():m_dir(""),m_loaded(false), m_nbUsers(1), m_noIdent(0)
{
	m_images.push_back(Image());
}

Anim::Anim(std::string dir):m_dir(dir), m_loaded(false), m_nbUsers(1)
{
	m_images.push_back(Image());
}

Anim::~Anim()
{
	m_imagesStr.clear();
}

bool Anim::testDir()
{
	bool dirExist=true;

	DIR *dp2;
	if((dp2  = opendir((m_dir).c_str())) == NULL) {
		dirExist=false;	
	}

	return dirExist;
}

void Anim::parseImages()
{
	ostringstream oss,oss2;
	DIR *dp2;
	struct dirent *dirp2;
	if((dp2  = opendir((m_dir).c_str())) != NULL) {
		while ((dirp2 = readdir(dp2)) != NULL) {
			if(dirp2->d_name[0]!='.' && fl_filename_match(dirp2->d_name, "*.png")!=0) {
				m_imagesStr.push_back(m_dir+"/"+dirp2->d_name);
			}
		}
	}
		

	//sort the images in the directory
	sort(m_imagesStr.begin(), m_imagesStr.end());
}

void Anim::testAction()
{
	switch(m_waitingAction) {
		case ANIM_ACTION_LOAD: {
			loadImages();
			m_waitingAction=ANIM_ACTION_NONE;
		}break;
		case ANIM_ACTION_UNLOAD: {
			unloadImages();
			m_waitingAction=ANIM_ACTION_NONE;
		}break;
		default:break;
	}
}

void Anim::loadImages()
{
	if(!m_loaded) {
		m_images.clear();
		//for each image of the anim
		for(unsigned int i=0;i<m_imagesStr.size();++i) {
			ostringstream oss2;
			oss2<<i;
			m_images.push_back(Image());
			Image& newImage = m_images[m_images.size()-1];
			if(newImage.load(m_imagesStr[i])) {
				m_images.pop_back();	
			}
		}

		//cout<<"Loaded "<<m_images.size()<<" images"<<endl;
		m_loaded=true;
	}
}

void Anim::unloadImages()
{
	if(m_loaded) {
		for(unsigned int i=0; i<m_images.size(); ++i) {
			Image& im = m_images[i];
			im.unload();
		}	
		m_images.clear();
		m_loaded=false;
	}
}

