/***************************************************************************
 *            Anim.h
 *
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef Anim_h
#define Anim_h

#include <vector>
#include <algorithm>
#include <string>
#include <iostream>
#include <sstream>
#include <cstring>
#include <stdio.h>
#include <dirent.h> 
#include <FL/filename.H>
#include "Image.hpp"

enum ANIM_ACTION {ANIM_ACTION_NONE, ANIM_ACTION_LOAD, ANIM_ACTION_UNLOAD};

class Anim  
{	
	public:
		Anim(); 
		Anim(std::string); 
		~Anim();

		inline std::string getDir(){return m_dir;}
		inline std::string getShortDir() {
			std::string shortDir=m_dir;
			if(shortDir[shortDir.size()-1]=='/') {
				shortDir=shortDir.substr(0,shortDir.size()-1);
			}
			size_t pos = shortDir.find_last_of("/"); 
			if(pos!=shortDir.npos) {
				shortDir=shortDir.substr(pos+1);	
			}
			return shortDir;
		}
		inline int getNbImagesStr(){return m_imagesStr.size();}
		inline int getNbImages(){return m_images.size();}
		//inline std::vector<Image>& getImages(){return m_images;}
		//inline Image& getImage(int im){return m_images[im];}
		inline const GLuint& getImageIdent(const unsigned int& im){
			if(im<m_images.size()) {
				return m_images[im].getIdent();
			} 
			return m_noIdent;
		}
		
		void parseImages();

		void testAction();
		void loadImages();
		void unloadImages();

		inline bool getLoaded(){return m_loaded;}

		inline void setAction(ANIM_ACTION action){m_waitingAction=action;}

		bool testDir();

		inline void addUser(){m_nbUsers++;}
		inline int removeUser(){m_nbUsers--;return m_nbUsers;}

	private:
		std::vector<std::string> m_imagesStr;
		std::vector<Image> m_images;
		std::string m_dir;
		bool m_loaded;
		int m_nbUsers;
		GLuint m_noIdent;

		ANIM_ACTION m_waitingAction;
};

#endif

