/***************************************************************************
 *            Element.cpp
 *
 *  2005  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Element.hpp"
using namespace std;

Element::Element():m_minValue(0),m_maxValue(1),m_value(0),m_anim(NULL), m_nullImage(NO_ANIM)
{
	m_colorMinValue[0]=0;
	m_colorMinValue[1]=0;
	m_colorMinValue[2]=0;
	m_colorMaxValue[0]=255;
	m_colorMaxValue[1]=255;
	m_colorMaxValue[2]=255;
	m_colorValue[0]=255;
	m_colorValue[1]=255;
	m_colorValue[2]=255;

}


Element::~Element()
{
}

const GLuint& Element::getImage()
{
	if(m_anim) {
		if(m_anim->getNbImages()>0) {
			if((int)m_value<m_anim->getNbImages() && m_value>=0) {	
				return m_anim->getImageIdent((int)m_value);
			}
			else{
				return m_anim->getImageIdent(m_anim->getNbImages()-1);
			}
		}
	}
	return m_nullImage;
}

const unsigned int& Element::processImage(const float& val)
{
	int imageVal = val*(m_maxValue-m_minValue)+m_minValue;
	if(m_anim) {
		if(m_anim->getNbImages()>0) {
			if(imageVal<m_anim->getNbImages() && m_value>=0) {	
				return m_anim->getImageIdent(imageVal);
				//return m_anim->getImages()[imageVal].getIdent();
			}
			else if(m_anim->getNbImages()>0){
				return m_anim->getImageIdent(m_anim->getNbImages()-1);
				//return m_anim->getImages()[m_anim->getNbImages()-1].getIdent();
			}
		}
	}
	return m_nullImage;
}

void Element::setColorRange(std::string& min, std::string& max)
{
	size_t pos;
	int cnt=0;
	pos = min.find_first_of(";");
	while(min.size()>0 && cnt<3) {
		std::istringstream is;
		is.str(min.substr(0,pos));
		is>>m_colorMinValue[cnt];
		min=min.substr(pos+1);
		pos = min.find_first_of(";");
		//cout<<"min val:"<<m_colorMinValue[cnt]<<endl;
		++cnt;
	}
	
	cnt=0;
	pos = max.find_first_of(";");
	while(max.size()>0 && cnt<3) {
		std::istringstream is;
		is.str(max.substr(0,pos));
		is>>m_colorMaxValue[cnt];
		max=max.substr(pos+1);
		pos = max.find_first_of(";");
		//cout<<"max val:"<<m_colorMaxValue[cnt]<<endl;;
		++cnt;
	}

}

void Element::setValue(const float& v)
{
	//m_valueChanged=true;

	m_colorValue[0]=v*float(m_colorMaxValue[0]-m_colorMinValue[0])+m_colorMinValue[0];
	m_colorValue[1]=v*float(m_colorMaxValue[1]-m_colorMinValue[1])+m_colorMinValue[1];
	m_colorValue[2]=v*float(m_colorMaxValue[2]-m_colorMinValue[2])+m_colorMinValue[2];
	m_value=v*(m_maxValue-m_minValue)+m_minValue;
}


