/***************************************************************************
 *            Element.h
 *
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef Element_h
#define Element_h

#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <stdio.h>

#include "Anim.hpp"

class Element  
{	
	public:
		Element(); 
		~Element();

		inline void setRange(const float& minVal, const float& maxVal) {
			m_minValue=minVal;
			m_maxValue=maxVal;
		}

		void setColorRange(std::string&, std::string&);
		inline const int & getColorValue(const int& c){return m_colorValue[c];}
		inline const int& processColorValue(const int& c, const int& val){m_processedColorValue[c]=val*float(m_colorMaxValue[c]-m_colorMinValue[c])+m_colorMinValue[c]; return m_processedColorValue[c];}
		inline const int & getColorMinValue(const int& c){return m_colorMinValue[c];}
		inline const int & getColorMaxValue(const int& c){return m_colorMaxValue[c];}
		inline void setColorRange(const int& c, const int& min, const int& max){m_colorMinValue[c]=min; m_colorMaxValue[c]=max;}

		inline const float& getValue(){return m_value;}
		inline const float& processValue(const float& val){m_processedValue = val*(m_maxValue-m_minValue)+m_minValue; return m_processedValue;}
		inline const float& getMinValue(){return m_minValue;}
		inline float& getMaxValue(){return m_maxValue;}

		void setValue(const float&);
		inline void setExactValue(const float& v){m_value=v;}
		inline void setAnim(Anim* anim){m_anim=anim;}
		inline Anim* getAnim(){return m_anim;}
		const GLuint& getImage();
		const unsigned int& processImage(const float&);


	private:
		float m_minValue;
		float m_maxValue;
		float m_value;
		float m_processedValue;
		Anim *m_anim;
		static const unsigned int NO_ANIM=0;
		unsigned int m_nullImage;

		int m_colorMinValue[3];
		int m_colorMaxValue[3];
		int m_colorValue[3];
		int m_processedColorValue[3];
};

#endif

