/***************************************************************************
 *            Grille.h
 *
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef Grille_h
#define Grille_h

#include <vector>
#include <string>
#include <iostream>
#include <stdio.h>
#include <dirent.h> 
#include <map>

#include "Parameter.hpp"
#include "Anim.hpp"

struct Vertex
{
	float x, y;
	Vertex(float x, float y)
	{
		this->x = x;
		this->y = y;
	}
};
struct VertexColor
{
	float r, g, b, a;
	VertexColor(float r, float g, float b, float a)
	{
		this->r = r;
		this->g = g;
		this->b = b;
		this->a = a;
	}
};
struct VertexTexture
{
	float x, y;// z;
	VertexTexture(float x, float y)
	{
		this->x = x;
		this->y = y;
		//this->z = b;
	}
};

struct VertexVectors
{
	std::vector<Vertex> m_vertices;
	std::vector<VertexColor> m_colors;
	std::vector<VertexTexture> m_textureCoords;
};

enum GRILLE_ORIGIN {ORIGIN_UP_LEFT, ORIGIN_UP_RIGHT, ORIGIN_DOWN_RIGHT, ORIGIN_DOWN_LEFT};
enum LOOP_STATE {LOOP_EMPTY, LOOP_RECORDING, LOOP_PLAYING, LOOP_STOPPED, LOOP_OVERDUBBING};
enum LOOP_PARAM {LOOP_TRIGGER, LOOP_ERASE, LOOP_SPEED, LOOP_TRANSPARENCY, LOOP_ERASE_TOGGLE, LOOP_PARAMS_NB};
enum LOOP_TRIGGER_MODE{LOOP_TRIGGER_NORMAL, LOOP_TRIGGER_OVERDUB};

class Grille  
{	
	public:
		Grille(); 
		~Grille();

		void copy(Grille&);

		inline Parameter& getParameter(const VJ_PARAM& p) {return m_params[p];}	
		inline void setParameterValue(const VJ_PARAM& p, const float& v) {m_params[p].setValue(v);}
		inline void setParameterInitValue(const VJ_PARAM& p, const float& v) {m_params[p].setInitValue(v);}
		inline void setParameterPatternPermutation(const VJ_PARAM& p, const float& v) {m_params[p].setPatternPermutation(v);}
		const float& getParameterValueAt(const VJ_PARAM&, const int&);
		inline const float& getParameterInitValue(const VJ_PARAM& p) {return m_params[p].getInitValue();}
		const GLuint& getImageAt(const int&);

		void grabInitValues();

		inline void setParameterPattern(const VJ_PARAM& param, const std::string& pattern) {m_params[param].setPattern(pattern);}

		inline const std::string& getName(){return m_name;}
		inline void setName(const std::string n){m_name=n;}

		inline const bool& getHomothety(){return m_homothety;}
		inline void setHomothety(const bool& h){m_homothety=h;}

		inline const GRILLE_ORIGIN& getOrigin(){return m_origin;}
		inline void setOrigin(const GRILLE_ORIGIN& o){m_origin=o;}

		inline const bool& getActive(){ return m_active;}
		inline void setActive(const bool& a){m_active=a;}
		
		void setAudioActive(const bool&);
		inline const bool& getAudioActive(){return m_audioActive;}

		void setElementAnim(const int&,Anim *);
		void setElementColorRange(const int&, std::string&, std::string&);

		//void removeAnims(){m_params[PARAM_IMAGE].removeAnims();}

		inline const bool& getBlend(){return m_blend;}
		inline void setBlend(const bool& blend){m_blend=blend;}

		void reinit();

		bool playLoop(const long&);
		bool triggerLoop(const LOOP_ACTION&, const long&, const LOOP_TRIGGER_MODE& mode=LOOP_TRIGGER_NORMAL);
		void eraseLoop();
		inline const LOOP_STATE& getLoopState(){return m_loopState;}
		inline void setLoopState(const LOOP_STATE& state){m_loopState=state;}
		void setLoopSpeed(const float&);
		inline void setLoopTransparency(const float& transp){m_loopTransparency=transp;}
		inline const float& getLoopTransparency(){return m_loopTransparency;}
		const float& getLoopPos();

		static const float MAX_ROWS=30;
		static const float MAX_COLS=30;

		inline std::map<GLuint, VertexVectors>& getGlCoords(){return m_glCoords;}
		inline const bool& hasGlCoords(){return m_hasGlCoords;}
		inline void hasGlCoords(const bool& h){m_hasGlCoords=h;}
		inline void clearGlCoords(){m_glCoords.clear();}

	private:
		std::string m_name;
		
		bool m_active;
		bool m_blend;
		float m_speed;

		bool m_homothety;
		GRILLE_ORIGIN m_origin;
		bool m_audioActive;

		//parameters
		Parameter m_params[PARAMS_NB];

		long m_loopStartTime;
		long m_loopDuration;
		float m_loopPos;
		
		LOOP_STATE m_loopState;
		float m_loopTransparency;

		//opengl coords
		std::map<GLuint, VertexVectors> m_glCoords;
		bool m_hasGlCoords;
};

#endif
