/***************************************************************************
 *            Image.cpp
 *
 *  2005  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Image.hpp"
#include "Vjpirate.hpp"

using namespace std;

Image::Image()
{

}


Image::~Image()
{
}

int Image::load(const std::string& im)
{
	int res=-1;
	Vjpirate *vjp = Vjpirate::getInstance();

	m_ident = vjp->getNewTextureID();

	glBindTexture( GL_TEXTURE_2D, m_ident);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	size_t size = im.size() + 1;
	char * buffer = new char[ size ];
	strncpy( buffer, im.c_str(), size );

	Fl_Shared_Image *newImage= Fl_Shared_Image::get(buffer);
	delete [] buffer;
	if(newImage->num_images()>0) {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
		gl_image(newImage,GL_TEXTURE_2D, 0, 0,GL_COMPRESSED_RGBA, newImage->w(), newImage->h() ); 	

		newImage->release();
		res=0;
	}


	return res;
}

void Image::unload()
{
	Vjpirate *vjp = Vjpirate::getInstance();
	vjp->freeTextureID(m_ident);
}

