/***************************************************************************
 *            Parameter.cpp
 *
 *  2005  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Parameter.hpp"
using namespace std;

Parameter::Parameter():m_value(0), m_initValue(0), m_patternPermut(0), m_loopStep(1), m_patternChanged(false)
{ 
	for(unsigned int i=0;i<2;++i) {
		m_loopIndex[i]=0;
		m_loopRecordAsked[i]=false;
		m_loopRecordTime[i]=0;
		m_loopPlayTime[i]=0;
	}
	m_elements.push_back(Element());
	m_pattern.push_back(0);
	m_patternStr = "A";
	m_initPatternStr = "A";


	m_names.push_back("cols");
	m_names.push_back("rows");
	m_names.push_back("size");
	m_names.push_back("rotation");
	m_names.push_back("color");
	m_names.push_back("transparency");
	m_names.push_back("image");
	m_names.push_back("displacementx");
	m_names.push_back("displacementy");
	m_names.push_back("permut_size");
	m_names.push_back("permut_rotation");
	m_names.push_back("permut_color");
	m_names.push_back("permut_transparency");
	m_names.push_back("permut_image");
	m_names.push_back("permut_displacementx");
	m_names.push_back("permut_displacementy");
	m_names.push_back("activation");
	m_names.push_back("reinit");
	m_names.push_back("blend");
	m_names.push_back("audio");
	

}


Parameter::~Parameter()
{
}

void Parameter::copy(Parameter& copiedParameter)
{
	//copy elements
	m_elements.clear();
	for(unsigned int i=0;i<copiedParameter.getNbElements();++i) {
		m_elements.push_back(copiedParameter.getElement(i));
	}

	//copy pattern
	setInitPattern(copiedParameter.getInitPatternStr());
	setPattern(copiedParameter.getPatternStr());
	m_patternPermut=copiedParameter.m_patternPermut;
	updatePattern();

	//copy init value
	setInitValue(copiedParameter.getInitValue());
	//copy value
	setValue(copiedParameter.getValue());
	
	//copy loop
	eraseLoop();

	for(unsigned int l=0;l<2;++l) {
		for(unsigned int i=0;i<copiedParameter.m_loopValues[l].size();++i) {
			m_loopValues[l].push_back(copiedParameter.m_loopValues[l][i]);
		}
		m_loopIndex[l]=copiedParameter.m_loopIndex[l];

		m_loopRecordTime[l]=copiedParameter.m_loopRecordTime[l];
		m_loopPlayTime[l]=copiedParameter.m_loopPlayTime[l];
		m_loopStep=copiedParameter.m_loopStep;
		m_loopState[l]=copiedParameter.m_loopState[l];
	}
}

void Parameter::setValue(const float& val, const VALUE_ORIGIN& origin)
{
	if(origin==EXTERN_VALUE && m_loopRecordAsked[VALUE_LOOP]) {
		m_loopState[VALUE_LOOP]=PARAMETER_LOOP_RECORDING;
		m_loopRecordAsked[VALUE_LOOP]=false;
	}

	//if recording, record in the new values buffer
	if(m_loopState[VALUE_LOOP]==PARAMETER_LOOP_RECORDING && origin==EXTERN_VALUE) {
		timeval curTime;
		gettimeofday(&curTime,NULL);
		long longCurTime = (long)(curTime.tv_sec-((int)((float)curTime.tv_sec/1000000.0)*1000000))*1000 + (long)((float)curTime.tv_usec/1000.0);
		LoopValue lv;
		lv.value=val;
		lv.time = longCurTime - m_loopRecordTime[VALUE_LOOP];
		m_newLoopValues[VALUE_LOOP].push_back(lv);
		
	}

	m_value=val;
	//change the value of the patternElements
	for(unsigned int i=0;i<m_elements.size();++i) {
		m_elements[i].setValue(val);
	}
}

void Parameter::grabInitValues()
{
	m_initValue=m_value;
	m_initPattern=m_pattern;
	m_initPatternStr=m_patternStr;
}

void Parameter::setInitValue(const float& val)
{
	m_initValue=val;
	setValue(val);
}

void Parameter::setPatternPermutation(const float& val, const VALUE_ORIGIN& origin)
{
	if(origin==EXTERN_VALUE && m_loopRecordAsked[PERMUT_LOOP]) {
		m_loopState[PERMUT_LOOP]=PARAMETER_LOOP_RECORDING;
		m_loopRecordAsked[PERMUT_LOOP]=false;
	}

	if(m_loopState[PERMUT_LOOP]==PARAMETER_LOOP_RECORDING && origin==EXTERN_VALUE) {
		timeval curTime;
		gettimeofday(&curTime,NULL);
		long longCurTime = (long)(curTime.tv_sec-((int)((float)curTime.tv_sec/1000000.0)*1000000))*1000 + (long)((float)curTime.tv_usec/1000.0);
		LoopValue lv;
		lv.value=m_value;
		lv.patternPermut=val;
		lv.time = longCurTime - m_loopRecordTime[PERMUT_LOOP];
		m_newLoopValues[PERMUT_LOOP].push_back(lv);
	}

	//set the pattern permut
	m_patternPermut=val;
	if(m_patternPermut>1) {
		m_patternPermut=1;
	}
	else if(m_patternPermut<0) {
		m_patternPermut=0;
	}
	
	double maxVal = pow((double)m_elements.size(),(double)m_pattern.size());	
	double doubleVal = val*(maxVal-1.0);
	doubleVal = floor(doubleVal);
	//cout<<"max et val "<<maxVal<<" "<<doubleVal<<endl;
	
	double doubleSize = double(m_elements.size());
	double d=0;
	for(int i=0;i<m_pattern.size();++i, ++d) {
		m_pattern[i] = long(fmod(doubleVal/pow(doubleSize,d),doubleSize));
		if(m_pattern[i]>=m_elements.size()) {
			m_pattern[i]=m_elements.size()-1;
		}
		m_patternStr[i]=ALPHA[m_pattern[i]];
	}

}

float Parameter::computePatternPermutation()
{

	//FIXME !!!

/*
	float maxVal = pow((float)m_elements.size(),(float)m_pattern.size());	
	float val;
	bool found=false;
	for(val=0;val<maxVal && !found;++val) {
		found=true;
		for(unsigned int i=0;i<m_pattern.size();++i) {
			if(m_pattern[i] != int(val/pow(m_elements.size(),i))%int(m_elements.size())) {
				found=false;
			}
		}
	}

	return val/maxVal;
*/
	return 0;
}

const float & Parameter::getValueAt(const int& pos)
{
	updatePattern();
	//get the modulo of the pos in the pattern, which gives the pattern element needed
	//then return the value of this pattern element
	return m_elements[m_pattern[pos%(m_pattern.size())]].getValue();
}

const float& Parameter::getInitValueAt(const int& pos)
{
	updatePattern();
	//get the modulo of the pos in the pattern, which gives the pattern element needed
	//then return the value of this pattern element
	return m_elements[m_initPattern[pos%(m_initPattern.size())]].processValue(m_initValue);
}

const GLuint& Parameter::getImageAt(const int& pos)
{
	updatePattern();
	//cout<<pos%m_pattern.size()<<" "<<m_pattern.size()<<" "<<m_elements.size()<<endl;
	return m_elements[m_pattern[pos%(m_pattern.size())]].getImage();
}

const GLuint& Parameter::getInitImageAt(const int& pos)
{
	updatePattern();
	return m_elements[m_initPattern[pos%(m_initPattern.size())]].processImage(m_initValue);
}

const int & Parameter::getColorValueAt(const int& pos, const int& color)
{
	updatePattern();
	return m_elements[m_pattern[pos%(m_pattern.size())]].getColorValue(color);
}

const int&  Parameter::getInitColorValueAt(const int& pos, const int& color)
{
	updatePattern();
	return m_elements[m_initPattern[pos%(m_initPattern.size())]].processColorValue(color,m_initValue);
}

void Parameter::setElementRange(const unsigned int& element, const float& minVal, const float& maxVal)
{
	if(element<m_elements.size()) {
		m_elements[element].setRange(minVal,maxVal);
		m_elements[element].setValue(m_value);
	}
}

void Parameter::setElementColorRange(const unsigned int& element, std::string& minVal, std::string& maxVal)
{
	if(element<m_elements.size()) {
		m_elements[element].setColorRange(minVal,maxVal);
		m_elements[element].setValue(m_value);
	}
}

void Parameter::setElementAnim(const unsigned int& element, Anim *anim)
{
	if(element<m_elements.size()) {
		m_elements[element].setAnim(anim);
	}
}

void Parameter::addElement()
{
	if(m_elements.size()<ALPHA.size())  {
		m_elements.push_back(Element());
		m_elements[m_elements.size()-1].setValue(m_value);
	}
}

void Parameter::removeLastElement()
{
	if(m_elements.size()>1) {
		m_elements.pop_back();
	}
}

void Parameter::addElement(const int& el)
{
	if(m_elements.size()<ALPHA.size() && el<ALPHA.size())  {
		m_elements.push_back(Element());
		m_elements[m_elements.size()-1].setRange(m_elements[el].getMinValue(),m_elements[el].getMaxValue());
		for(int i=0;i<3;++i) {
			m_elements[m_elements.size()-1].setColorRange(i,m_elements[el].getColorMinValue(i),m_elements[el].getColorMaxValue(i));
		}
		m_elements[m_elements.size()-1].setAnim(m_elements[el].getAnim());
		m_elements[m_elements.size()-1].setValue(m_value);
	}
}

void Parameter::removeElement(const int& el)
{
	if(m_elements.size()>1 && el<ALPHA.size()) {
		m_elements.erase(m_elements.begin()+el);
	}
}

int upper(int c)
{
  return std::toupper((unsigned char)c);
}

void Parameter::setInitPattern(const std::string& newIpat)
{
	std::string ipat = newIpat;
	//if not null
	if(ipat.size()>0) {
		//convert to uppercase
		std::transform(ipat.begin(), ipat.end(), ipat.begin(), upper);
		//test if pattern elements exist or remove them from new pattern
		for(unsigned int i=0;i<ipat.size();) {
			size_t pos = ALPHA.find_first_of(ipat[i]);
			if(pos==ALPHA.npos || pos>=m_elements.size()) {
				ipat.erase(ipat.begin()+i);
			}
			else {
				++i;
			}
		}
		if(ipat.size()>0) {
			m_initPatternStr=ipat;
			m_initPattern.resize(ipat.size(),0);
			for(unsigned int i=0;i<ipat.size();++i) {
				size_t pos = ALPHA.find_first_of(ipat[i]);
				if(pos!=ALPHA.npos) {
					m_initPattern[i]=(int)pos;
				}
			}
			setPattern(ipat);
		}
	}
}

void Parameter::setPattern(const std::string& patt)
{
	if(patt.size()>0) {
		m_newPattern.resize(patt.size(),0);
		for(unsigned int i=0;i<patt.size();++i) {
			size_t pos = ALPHA.find_first_of(patt[i]);
			if(pos!=ALPHA.npos) {
				m_newPattern[i]=(int)pos;
			}
		}
		m_patternChanged=true;
		m_patternStr=patt;
		m_patternPermut=computePatternPermutation();
	}
}


void Parameter::startLoopRecording(const long& curTime)
{
	for(unsigned int loop=0;loop<2;++loop) {
		//if the parameter does not have an audio input, add a first value and start recording
		m_newLoopValues[loop].clear();
		LoopValue lv;
		lv.value=m_value;
		lv.time=0;
		lv.patternPermut=m_patternPermut;
		m_newLoopValues[loop].push_back(lv);
		m_loopRecordAsked[loop]=true;
		m_loopRecordTime[loop]=curTime;
	}
}

void Parameter::stopLoopRecording(const long& curTime)
{
	for(unsigned int loop=0;loop<2;++loop) {
		if(m_newLoopValues[loop].size()>1) {
			//add a final Event
			LoopValue lv;
			lv.value=m_value;

			lv.time=curTime-m_loopRecordTime[loop];

			m_newLoopValues[loop].push_back(lv);

			m_loopValues[loop] = m_newLoopValues[loop];

			startLoopPlaying(curTime);
		}

		m_newLoopValues[loop].clear();
		if(m_loopValues[loop].size()>1) {
			m_loopState[loop]=PARAMETER_LOOP_PLAYING;
		}
		else {
			m_loopState[loop]=PARAMETER_LOOP_NONE;
		}
		m_loopRecordAsked[loop]=false;
	}
}

void Parameter::startLoopPlaying(const long& curTime)
{
	for(unsigned int loop=0;loop<2;++loop) {
		if(m_loopValues[loop].size()>0) {
			m_loopPlayTime[loop]=curTime;
			m_loopIndex[loop]=0;	
			m_loopState[loop]=PARAMETER_LOOP_PLAYING;
		}
	}
}

void Parameter::eraseLoop()
{
	for(unsigned int i=0;i<2;++i) {
		m_loopValues[i].clear();
		m_newLoopValues[i].clear();
		m_loopState[i]=PARAMETER_LOOP_NONE;
	}
}

bool Parameter::playLoop(const long& curTime)
{
	bool changed=false;
	
	for(unsigned int loop=0;loop<2;++loop) {
		bool loopEnd=false;
		if(m_loopState[loop]==PARAMETER_LOOP_PLAYING) {
			bool eventBeforeCurTime=true;
			while(eventBeforeCurTime && m_loopIndex[loop]<m_loopValues[loop].size()) {
				if((long)((float)(m_loopValues[loop][m_loopIndex[loop]].time)/m_loopStep)<=curTime-m_loopPlayTime[loop]) {
					changed=true;
					if(LOOP(loop)==VALUE_LOOP) {
						setValue(m_loopValues[loop][m_loopIndex[loop]].value, LOOP_VALUE);
					} else { //PERMUT LOOP
						if(m_loopValues[loop][m_loopIndex[loop]].patternPermut!=m_patternPermut) {
							setPatternPermutation(m_loopValues[loop][m_loopIndex[loop]].patternPermut, LOOP_VALUE);
						}
					}
					m_loopIndex[loop]++;
				}
				else {
					//cout<<"after"<<endl;
					eventBeforeCurTime=false;
				}
			}
			//if reached the end of the loop
			if(m_loopIndex[loop]>=m_loopValues[loop].size()) {
				m_loopPlayTime[loop] = curTime;
				m_loopIndex[loop] = 0;
			}
		}
	}


	return changed;
}

void Parameter::getIdFromName(const std::string& name, int& id)
{
	int i=0;
	while(i<m_names.size() && m_names[i].compare(name)!=0) {
		++i;
	}
	id=i;
}

void Parameter::getNameFromId(const int& id, std::string &name)
{
	if(id>=0 && id<m_names.size()) {
		name=m_names[id];		
	}
	else {
		name=m_names[0];
	}
}

