/***************************************************************************
 *            Parameter.h
 *
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef Parameter_h
#define Parameter_h

#include <vector>
#include <string>
#include <iostream>
#include <stdio.h>
#include <sys/time.h>
#include <cmath>

#include "Element.hpp"

enum VJ_PARAM{PARAM_COLS, PARAM_ROWS, PARAM_SIZE, PARAM_ROTATION, PARAM_COLOR, PARAM_TRANS, PARAM_IMAGE, PARAM_DISX, PARAM_DISY, PARAMS_NB};
enum VJ_PERMUT_PARAM{PERMUT_PARAM_SIZE, PERMUT_PARAM_ROTATION, PERMUT_PARAM_COLOR, PERMUT_PARAM_TRANS, PERMUT_PARAM_IMAGE, PERMUT_PARAM_DISX, PERMUT_PARAM_DISY, PERMUT_PARAMS_NB};
//enum VJ_PARAM{PARAM_COLS, PARAM_ROWS, PARAM_SIZE, PARAM_ROTATION, PARAM_COLOR, PARAM_TRANS, PARAM_IMAGE, PARAMS_NB};
//enum VJ_PERMUT_PARAM{PERMUT_PARAM_SIZE, PERMUT_PARAM_ROTATION, PERMUT_PARAM_COLOR, PERMUT_PARAM_TRANS, PERMUT_PARAM_IMAGE, PERMUT_PARAMS_NB};
enum VJ_ADD_PARAM{PARAM_ACTIVATION, PARAM_REINIT, PARAM_BLEND, PARAM_AUDIO, ADD_PARAMS_NB};
static const std::string ALPHA = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

enum OVERDUB_STATE{OVERDUB_NONE, OVERDUB_ASKED, OVERDUB_STARTED};
enum VALUE_ORIGIN{LOOP_VALUE,EXTERN_VALUE,AUDIO_VALUE};
enum LOOP{VALUE_LOOP, PERMUT_LOOP};
enum LOOP_ACTION{LOOP_ACTION_TRIGGER, LOOP_ACTION_PLAY, LOOP_ACTION_STOP};

enum PARAMETER_LOOP_STATE{PARAMETER_LOOP_NONE, PARAMETER_LOOP_RECORDING, PARAMETER_LOOP_PLAYING};

struct LoopValue
{
	float value;
	long time;
	float patternPermut;
};


class Parameter  
{	
	public:
		Parameter(); 
		~Parameter();
	
		void copy(Parameter&);

		void setValue(const float&, const VALUE_ORIGIN& origin=EXTERN_VALUE);
		void setInitValue(const float&);
		inline const float& getInitValue(){return m_initValue;}
		inline const float& getValue(){return m_value;}
		
		void grabInitValues();

		void setElementRange(const unsigned int&, const float&, const float&);
		void setElementColorRange(const unsigned int&, std::string&, std::string&);
		void setElementAnim(const unsigned int&,Anim*);
		void addElement();
		void removeLastElement();
		void addElement(const int&);
		void removeElement(const int&);
		inline const float& getFirstElementValue() {return m_elements[0].getValue();}
		inline Element& getElement(int el){return m_elements[el];}
		inline unsigned int getNbElements(){return m_elements.size();}

		void setPattern(const std::string&);
		void setInitPattern(const std::string&);
		void setPatternPermutation(const float& val, const VALUE_ORIGIN& origin=EXTERN_VALUE);
		inline const float& getPatternPermutation(){return m_patternPermut;}
		float computePatternPermutation();
		inline const std::string& getPatternStr(){return m_patternStr;}
		inline void reinitPattern(){setPattern(m_initPatternStr);}
		inline const std::string& getInitPatternStr(){return m_initPatternStr;}
		inline void updatePattern(){if(m_patternChanged){m_pattern=m_newPattern;m_patternChanged=false;} }


		const float& getValueAt(const int&);
		const float& getInitValueAt(const int&);
		const GLuint& getImageAt(const int&);
		const GLuint& getInitImageAt(const int&);
		const int& getColorValueAt(const int&, const int&);
		const int& getInitColorValueAt(const int&, const int&);
		
		void startLoopRecording(const long&);
		void stopLoopRecording(const long&);
		void startLoopPlaying(const long&);
		void eraseLoop();
		inline void setSpeed(const float& speed){m_loopStep=speed;}
		bool playLoop(const long&);
		inline int getNbLoopValues() { return m_loopValues[VALUE_LOOP].size();}
		inline const LoopValue& getLoopValue(const int& lv) { return m_loopValues[VALUE_LOOP][lv];}
		inline int getNbLoopPermut() { return m_loopValues[PERMUT_LOOP].size();}
		inline const LoopValue& getLoopPermut(const int& lv) { return m_loopValues[PERMUT_LOOP][lv];}
		
		void getIdFromName(const std::string&, int&);
		void getNameFromId(const int&, std::string&);
	
	private:

		float m_value;
		float m_initValue;
		std::vector<Element> m_elements;
		std::vector<int> m_pattern;	
		std::string m_patternStr;
		float m_patternPermut;
		std::string m_initPatternStr;
		std::vector<int> m_initPattern;	
		std::vector<int> m_newPattern;
		bool m_patternChanged;

		std::vector<LoopValue> m_loopValues[2];
		long m_loopIndex[2];
		long m_loopPlayTime[2];
		long m_loopRecordTime[2];
		float m_loopStep;

		bool m_loopRecordAsked[2];
		PARAMETER_LOOP_STATE m_loopState[2];

		std::vector<LoopValue> m_newLoopValues[2];
		
		std::vector<std::string> m_names;

};

#endif

