/***************************************************************************
 *            ExecWindow.cpp
 *
 *  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <math.h>
#include <unistd.h>

#include "ExecWindow.hpp"
#include "../base/Vjpirate.hpp"


using namespace std;


ExecWindow::ExecWindow(int w,int h,const char* titre,Vjpirate *t):Fl_Double_Window(w,h,titre), m_vj(t), m_fileSaved(true), m_refreshAllPreviews(false), m_openFile(false)
{
	//default theme
	Fl::set_color(FL_BACKGROUND_COLOR,0,0,0);
	Fl::set_color(FL_BACKGROUND2_COLOR,80,80,80);
	Fl::set_color(FL_FOREGROUND_COLOR,255,255,255);
	Fl::set_color(FL_INACTIVE_COLOR,40,40,40);
	Fl::set_color(FL_SELECTION_COLOR,0,0,255);

	//resizable
	//resizable(this);
	
	//borders off
	//border(false);
	
	//set window icon
	/*
	fl_open_display();
	Pixmap p, mask;
	XpmCreatePixmapFromData(fl_display, DefaultRootWindow(fl_display), vjpirate_xpm, &p, &mask, NULL);
	icon((char *)p);
	*/

	//callback
	this->callback(statWindow,this);

	m_windowBar = new Fl_Group(0,0,w,MENU_HEIGHT,"");
	m_windowBar->color(FL_INACTIVE_COLOR);
	m_windowBar->box(FL_FLAT_BOX);
	m_windowBar->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
	m_windowBar->labelcolor(FL_BACKGROUND2_COLOR);
	m_windowBar->labelsize(20);
	this->add(m_windowBar);

	//menu
	Fl_Menu_Item menuitems[] = 
	{
		{ "VJP",0, 0, 0, FL_SUBMENU },
			{ "&Open File",FL_CTRL + 'o',(Fl_Callback *)statOpen,this },
			{ "&Import Inputs",FL_CTRL + 'i',(Fl_Callback *)statImportInputs,this },
			{ "&Save File",FL_CTRL + 's',(Fl_Callback *)statSave,this},
			{ "S&ave File as",FL_CTRL + FL_SHIFT + 's',(Fl_Callback *)statSaveAs,this},
			{ "&Quit",FL_CTRL + 'q', (Fl_Callback *)statQuit,this},
			{0},
		{0}
	};
	m_menu = new Fl_Menu_Bar(0, WIDGETS_SEP, 50, 15);
	m_menu->copy(menuitems);
	m_menu->box(FL_NO_BOX);
	m_menu->textcolor(FL_BACKGROUND2_COLOR);
	m_menu->selection_color(FL_SELECTION_COLOR);
	m_menu->color(FL_INACTIVE_COLOR);
	m_menu->clear_visible_focus();
	m_windowBar->add(m_menu);


	//MAPPINGS MENU
	Fl_Menu_Item learnMenuItems[] = {
		{ "Learn Input",0,0,this },
		{ "Add Audio Input",0,0,this},
		{0},
	};
	m_learnMenu = new Fl_Menu_Button(0, 0, 50, MENU_HEIGHT);
	m_learnMenu->copy(learnMenuItems);
	m_learnMenu->textsize(10);
	m_learnMenu->type(Fl_Menu_Button::POPUP123);
	m_learnMenu->hide();
	m_learnMenu->callback(statLearnMenu,this);
	this->add(m_learnMenu);

	//GRILLE LIST
	m_execGrilleScroll = new GrilleScroll(0, MENU_HEIGHT, w, 140,"Grids");
	//m_execGrilleScroll = new Fl_Scroll(0, MENU_HEIGHT, w, 140,"Grids");
	m_execGrilleScroll->align(FL_ALIGN_TOP|FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
	m_execGrilleScroll->type(Fl_Scroll::HORIZONTAL_ALWAYS);
	m_execGrilleScroll->box(FL_FLAT_BOX);
	m_execGrilleScroll->color(FL_BACKGROUND2_COLOR);
	m_execGrilleScroll->color2(FL_BACKGROUND2_COLOR);
	m_execGrilleScroll->selection_color(FL_BACKGROUND2_COLOR);
	m_execGrilleScroll->callback(statExecGrillesScroll,this);
	this->add(m_execGrilleScroll);
	m_addGrille = new Fl_Button(50,MENU_HEIGHT+3,10,10,"+");
	m_addGrille->box(FL_FLAT_BOX);
	m_addGrille->clear_visible_focus();
	m_addGrille->color(FL_BACKGROUND2_COLOR);
	m_addGrille->color2(FL_SELECTION_COLOR);
	m_addGrille->callback(statAddGrille,this);
	this->add(m_addGrille);

	////MONITOR
	m_monitor = new RenderWindow(50, m_execGrilleScroll->y()+m_execGrilleScroll->h()+WIDGETS_SEP*2, 500, 375, "Monitor", RENDER_MONITOR);	
	m_monitor->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	this->add(m_monitor);
    this->resizable(m_monitor);

	//MASTER TRANSPARENCY
	m_execMasterSlider= new TardiSlider(m_monitor->x()+m_monitor->w()+70, m_monitor->y()+40, EXEC_PARAMS_W, EXEC_PARAMS_H, "Master");
	m_execMasterSlider->callback(statExecMaster,this);
	m_execMasterSlider->bounds(0.0,1.0);
	m_execMasterSlider->value(1.0);;
	this->add(m_execMasterSlider);

	//SELECTION ACTIVATION
	m_execSelectActButton = new TardiButton(m_execMasterSlider->x()+m_execMasterSlider->w()+20, m_execMasterSlider->y(), 50, EXEC_PARAMS_H, "Sel Act");
	m_execSelectActButton->box(FL_RFLAT_BOX);
	m_execSelectActButton->value(1);
	m_execSelectActButton->type(FL_TOGGLE_BUTTON);
	m_execSelectActButton->down_box(FL_RFLAT_BOX);
	m_execSelectActButton->selection_color(FL_SELECTION_COLOR);
	m_execSelectActButton->color(FL_BACKGROUND2_COLOR);
	m_execSelectActButton->clear_visible_focus();
	m_execSelectActButton->callback(statExecSelectAct,this);
	this->add(m_execSelectActButton);

	//
	m_execSeparator = new Fl_Box(m_monitor->x()+m_monitor->w()+70, m_execMasterSlider->y()+EXEC_PARAMS_H*2, 250, WIDGETS_SEP/2);
	m_execSeparator->box(FL_FLAT_BOX);
	m_execSeparator->color(FL_INACTIVE_COLOR);
	this->add(m_execSeparator);


	//grille group
	m_execGrilleGroup = new Fl_Group(m_monitor->x()+m_monitor->w()+70, m_monitor->y()+70, 300, 325 ,"CURRENT GRILLE");
	m_execGrilleGroup->end();
	m_execGrilleGroup->align(FL_ALIGN_TOP|FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
	m_execGrilleGroup->labelcolor(FL_INACTIVE_COLOR);
	m_execGrilleGroup->box(FL_BORDER_BOX);
	m_execGrilleGroup->color(FL_BLACK);
	this->add(m_execGrilleGroup);


	////GRILLE GROUP
	int paramPosX=m_execGrilleGroup->x()+15;
	m_execActButton = new TardiButton(paramPosX,m_execGrilleGroup->y()+(EXEC_PARAMS_Y/5*4),60,15,"Active");
	m_execActButton->box(FL_RFLAT_BOX);
	m_execActButton->type(FL_TOGGLE_BUTTON);
	m_execActButton->down_box(FL_RFLAT_BOX);
	m_execActButton->selection_color(FL_SELECTION_COLOR);
	m_execActButton->color(FL_BACKGROUND2_COLOR);
	m_execActButton->clear_visible_focus();
	m_execActButton->callback(statExecActButton,this);
	m_execGrilleGroup->add(m_execActButton);

	m_execAudioButton = new TardiButton(m_execActButton->x()+m_execActButton->w()+20,m_execActButton->y(),50,15,"Audio");
	m_execAudioButton->box(FL_RFLAT_BOX);
	m_execAudioButton->type(FL_TOGGLE_BUTTON);
	m_execAudioButton->down_box(FL_RFLAT_BOX);
	m_execAudioButton->selection_color(FL_SELECTION_COLOR);
	m_execAudioButton->color(FL_BACKGROUND2_COLOR);
	m_execAudioButton->clear_visible_focus();
	m_execAudioButton->value(0);
	m_execAudioButton->callback(statExecAudioButton,this);
	m_execGrilleGroup->add(m_execAudioButton);

	m_execInitValuesButton = new TardiButton(paramPosX+EXEC_PARAMS_X,m_execActButton->y(),60,15,"Init");
	m_execInitValuesButton->box(FL_RFLAT_BOX);
	m_execInitValuesButton->down_box(FL_RFLAT_BOX);
	m_execInitValuesButton->selection_color(FL_SELECTION_COLOR);
	m_execInitValuesButton->color(FL_BACKGROUND2_COLOR);
	m_execInitValuesButton->labelcolor(FL_WHITE);
	m_execInitValuesButton->clear_visible_focus();
	m_execInitValuesButton->callback(statExecInitValuesButton,this);
	m_execGrilleGroup->add(m_execInitValuesButton);


	//FIXME 
	m_editGrabInitValue = new Fl_Button(m_execInitValuesButton->x()+70,m_execInitValuesButton->y(),20,20,"<-");
	m_editGrabInitValue->box(FL_BORDER_BOX);
	m_editGrabInitValue->selection_color(FL_SELECTION_COLOR);
	m_editGrabInitValue->clear_visible_focus();
	m_editGrabInitValue->callback(statExecInitValuesButton,this);
	m_execGrilleGroup->add(m_editGrabInitValue);

	//param sliders
	/*
	m_execParamSliders[PARAM_COLS] = new TardiSlider(paramPosX, m_execGrilleGroup->y()+2*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_ROWS] = new TardiSlider(paramPosX+EXEC_PARAMS_X, m_execGrilleGroup->y()+2*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_SIZE] = new TardiSlider(paramPosX, m_execGrilleGroup->y()+3*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_TRANS] = new TardiSlider(paramPosX+EXEC_PARAMS_X, m_execGrilleGroup->y()+3*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_ROTATION] = new TardiSlider(paramPosX, m_execGrilleGroup->y()+4*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_IMAGE] = new TardiSlider(paramPosX+EXEC_PARAMS_X, m_execGrilleGroup->y()+4*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_COLOR] = new TardiSlider(paramPosX, m_execGrilleGroup->y()+5*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_DISX] = new TardiSlider(paramPosX+EXEC_PARAMS_X, m_execGrilleGroup->y()+5*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	m_execParamSliders[PARAM_DISY] = new TardiSlider(paramPosX, m_execGrilleGroup->y()+6*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");
	*/
	for(unsigned int i=0;i<PARAMS_NB;++i) {

		m_execParamSliders[i] = new TardiSlider(paramPosX+(i%2)*EXEC_PARAMS_X, m_execGrilleGroup->y()+(i/2+2)*EXEC_PARAMS_Y, EXEC_PARAMS_W, EXEC_PARAMS_H,"");

		m_execParamSliders[i]->callback(statExecParamValueSliders,this);
		m_execParamSliders[i]->bounds(0.0,1.0);
		m_execParamSliders[i]->value(0.0);
		m_execParamAudio[i] = new Fl_Button(m_execParamSliders[i]->x()-15, m_execParamSliders[i]->y(), 15,EXEC_PARAMS_H,"@<|");
		m_execParamAudio[i]->box(FL_NO_BOX);
		m_execParamAudio[i]->labelsize(EXEC_PARAMS_H);
		m_execParamAudio[i]->clear_visible_focus();
		m_execParamNames[i] = new Fl_Toggle_Button(m_execParamSliders[i]->x(), m_execParamSliders[i]->y()-2*EXEC_PARAMS_H, EXEC_PARAMS_W, EXEC_PARAMS_H*2,"");
		m_execParamNames[i]->box(FL_FLAT_BOX);
		m_execParamNames[i]->down_box(FL_FLAT_BOX);
		m_execParamNames[i]->color2(FL_BACKGROUND2_COLOR);
		m_execParamNames[i]->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
		m_execParamNames[i]->clear_visible_focus();
		m_execParamNames[i]->callback(statParamNames, this);
		m_execGrilleGroup->add(m_execParamSliders[i]);
		m_execGrilleGroup->add(m_execParamAudio[i]);
		m_execGrilleGroup->add(m_execParamNames[i]);
	}
	m_execParamNames[PARAM_COLS]->copy_label("Columns");
	m_execParamNames[PARAM_ROWS]->copy_label("Rows");
	m_execParamNames[PARAM_SIZE]->copy_label("Size");
	m_execParamNames[PARAM_TRANS]->copy_label("Transparency");
	m_execParamNames[PARAM_ROTATION]->copy_label("Rotation");
	m_execParamNames[PARAM_IMAGE]->copy_label("Image");
	m_execParamNames[PARAM_COLOR]->copy_label("Color");
	m_execParamNames[PARAM_DISX]->copy_label("Displacement X");
	m_execParamNames[PARAM_DISY]->copy_label("Displacement Y");

	for(unsigned int i=0;i<PERMUT_PARAMS_NB;++i) {
		m_execParamPermut[i] = new PatternValue(m_execParamSliders[i+2]->x(), m_execParamSliders[i+2]->y()+EXEC_PARAMS_H, EXEC_PARAMS_W,EXEC_PARAMS_H,"A");
		m_execParamPermut[i]->callback(statExecParamPermut,this);
		m_execParamPermut[i]->bounds(0.0,1.0);
		m_execParamPermut[i]->value(0.0);
		m_execParamPermut[i]->soft(0);
		m_execParamPermut[i]->step(0.01);
		m_execParamPermutAudio[i] = new Fl_Button(m_execParamSliders[i+2]->x()-15, m_execParamSliders[i+2]->y()+EXEC_PARAMS_H, 15,EXEC_PARAMS_H,"@<|");
		m_execParamPermutAudio[i]->box(FL_NO_BOX);
		m_execParamPermutAudio[i]->labelsize(EXEC_PARAMS_H);
		m_execParamPermutAudio[i]->clear_visible_focus();
		m_execGrilleGroup->add(m_execParamPermut[i]);
		m_execGrilleGroup->add(m_execParamPermutAudio[i]);
	}
	m_execBlendButton = new TardiButton(paramPosX+EXEC_PARAMS_X, m_execGrilleGroup->y()+6*EXEC_PARAMS_Y,50,15,"Blend");
	m_execBlendButton->box(FL_RFLAT_BOX);
	m_execBlendButton->type(FL_TOGGLE_BUTTON);
	m_execBlendButton->down_box(FL_RFLAT_BOX);
	m_execBlendButton->selection_color(FL_SELECTION_COLOR);
	m_execBlendButton->color(FL_BACKGROUND2_COLOR);
	m_execBlendButton->clear_visible_focus();
	m_execBlendButton->callback(statExecBlendButton,this);
	m_execGrilleGroup->add(m_execBlendButton);


	//EDIT GROUPS
	////EDIT PROPORTIONS
	m_editProportionsGroup=new Fl_Group(0, 0, 250,200,"");
	m_editProportionsGroup->box(FL_FLAT_BOX);
	m_editProportionsGroup->color(FL_BACKGROUND2_COLOR);
	m_editOrigin = new Fl_Group(m_editProportionsGroup->x()+WIDGETS_SEP, m_editProportionsGroup->y()+100, 60, 60, "Origin"),
	m_editOrigin->box(FL_FLAT_BOX);
	m_editOrigin->color(FL_BACKGROUND2_COLOR);
	m_editOrigin->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editOrigin->end();
	m_editProportionsGroup->add(m_editOrigin);
	m_editRows = new Fl_Value_Input(m_editProportionsGroup->x()+50, m_editProportionsGroup->y()+WIDGETS_SEP, 50, 20, "Rows");	
	m_editRows->callback(statEditProportions,this);
	m_editRows->step(1);
	m_editRows->bounds(1,Grille::MAX_ROWS);
	m_editProportionsGroup->add(m_editRows);
	m_editCols = new Fl_Value_Input(m_editProportionsGroup->x()+180, m_editProportionsGroup->y()+WIDGETS_SEP, 50, 20, "Columns");	
	m_editCols->callback(statEditProportions,this);
	m_editCols->step(1);
	m_editCols->bounds(1,Grille::MAX_ROWS);
	m_editProportionsGroup->add(m_editCols);
	m_editHomothety = new Fl_Check_Button(m_editProportionsGroup->x()+90, m_editProportionsGroup->y()+50, 15, 15, "Homothety");
	m_editHomothety->align(FL_ALIGN_LEFT);
	m_editHomothety->callback(statEditProportions,this);
	m_editProportionsGroup->add(m_editHomothety);
	m_editOriginButtons[0] = new Fl_Check_Button(m_editOrigin->x(),m_editOrigin->y(),15,15,"");
	m_editOriginButtons[1] = new Fl_Check_Button(m_editOrigin->x()+m_editOrigin->w()-15,m_editOrigin->y(),15,15,"");
	m_editOriginButtons[2] = new Fl_Check_Button(m_editOrigin->x()+m_editOrigin->w()-15,m_editOrigin->y()+m_editOrigin->h()-15,15,15,"");
	m_editOriginButtons[3] = new Fl_Check_Button(m_editOrigin->x(),m_editOrigin->y()+m_editOrigin->h()-15,15,15,"");
	for(unsigned int i=0;i<4;++i) {
		m_editOriginButtons[i]->type(FL_RADIO_BUTTON);
		m_editOriginButtons[i]->callback(statEditOrigin,this);
		m_editOrigin->add(m_editOriginButtons[i]);
	}
	m_editProportionsGroup->hide();

	////EDIT PARAMETERS
	m_editParameterGroup=new Fl_Group(0, 0, 260,200,"");
	m_editParameterGroup->box(FL_FLAT_BOX);
	m_editParameterGroup->color(FL_BACKGROUND2_COLOR);
	m_editElements = new Fl_Scroll(m_editParameterGroup->x()+WIDGETS_SEP, m_editParameterGroup->y()+WIDGETS_SEP, m_editParameterGroup->w()-2*WIDGETS_SEP, 100,"");
	m_editElements->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editElements->type(Fl_Scroll::VERTICAL_ALWAYS);
	m_editElements->box(FL_BORDER_BOX);
	m_editParameterGroup->add(m_editElements);
	m_editPattern = new Fl_Input(m_editElements->x() + WIDGETS_SEP, m_editElements->y()+m_editElements->h()+30, 120, 30, "Pattern");
	m_editPattern->align(FL_ALIGN_LEFT|FL_ALIGN_TOP);
	m_editPattern->callback(statEditPattern,this);
	m_editPattern->when(FL_WHEN_CHANGED);
	m_editParameterGroup->add(m_editPattern);
	m_editInitValue = new Fl_Value_Input(m_editPattern->x()+70, m_editPattern->y()+m_editPattern->h()+WIDGETS_SEP, 30, 20, "Init Value");
	m_editInitValue->align(FL_ALIGN_LEFT);
	m_editInitValue->callback(statEditInitValue,this);
	m_editParameterGroup->add(m_editInitValue);

	for(unsigned int i=0;i<26;++i) {
		m_editElementsIndex[i] = new Fl_Output(m_editElements->x()+WIDGETS_SEP,m_editElements->y()+WIDGETS_SEP+(i+1)*20,20,20,"");
		m_editElementsIndex[i]->value(ALPHA.substr(i,1).c_str());
		m_editElementsIndex[i]->box(FL_BORDER_BOX);
		m_editElementsIndex[i]->deactivate();
		m_editElementsIndex[i]->hide();
		m_editElements->add(m_editElementsIndex[i]);
		m_editElementsMinFloat[i] = new Fl_Value_Input(m_editElements->x()+2*WIDGETS_SEP+20,m_editElementsIndex[i]->y(),30,20,"");
		m_editElementsMinFloat[i]->hide();
		m_editElementsMinFloat[i]->callback(statEditElements,this);
		m_editElementsMinFloat[i]->when(FL_WHEN_RELEASE);
		m_editElements->add(m_editElementsMinFloat[i]);
		m_editElementsMaxFloat[i] = new Fl_Value_Input(m_editElements->x()+3*WIDGETS_SEP+50,m_editElementsIndex[i]->y(),30,20,"");
		m_editElementsMaxFloat[i]->callback(statEditElements,this);
		m_editElementsMaxFloat[i]->when(FL_WHEN_RELEASE);
		m_editElementsMaxFloat[i]->hide();
		m_editElements->add(m_editElementsMaxFloat[i]);
		m_editElementsMinBut[i] = new Fl_Button(m_editElements->x()+2*WIDGETS_SEP+20,m_editElementsIndex[i]->y(),30,20,"");
		m_editElementsMinBut[i]->callback(statEditElements,this);
		m_editElementsMinBut[i]->hide();
		m_editElements->add(m_editElementsMinBut[i]);
		m_editElementsMaxBut[i] = new Fl_Button(m_editElements->x()+3*WIDGETS_SEP+50,m_editElementsIndex[i]->y(),30,20,"");
		m_editElementsMaxBut[i]->callback(statEditElements,this);
		m_editElementsMaxBut[i]->hide();
		m_editElements->add(m_editElementsMaxBut[i]);
		m_editElementsImBut[i] = new Fl_Button(m_editElements->x()+4*WIDGETS_SEP+70,m_editElementsIndex[i]->y(),90,20,"");
		m_editElementsImBut[i]->callback(statEditElements,this);
		m_editElementsImBut[i]->label("Images");
		m_editElementsImBut[i]->hide();
		m_editElements->add(m_editElementsImBut[i]);
		m_editElementsDup[i]= new Fl_Button(m_editElements->x()+m_editElements->w()-50 , m_editElements->y()+WIDGETS_SEP+(i+1)*20, 15, 15,"+");
		m_editElementsDup[i]->hide();
		m_editElementsDup[i]->box(FL_BORDER_BOX);
		m_editElementsDup[i]->selection_color(FL_SELECTION_COLOR);
		m_editElementsDup[i]->clear_visible_focus();
		m_editElementsDup[i]->callback(statEditElements,this);
		m_editElements->add(m_editElementsDup[i]);
		m_editElementsDel[i]= new Fl_Button(m_editElements->x()+m_editElements->w()-35, m_editElements->y()+WIDGETS_SEP+(i+1)*20, 15, 15,"-");
		m_editElementsDel[i]->hide();
		m_editElementsDel[i]->box(FL_BORDER_BOX);
		m_editElementsDel[i]->selection_color(FL_SELECTION_COLOR);
		m_editElementsDel[i]->clear_visible_focus();
		m_editElementsDel[i]->callback(statEditElements,this);
		m_editElements->add(m_editElementsDel[i]);
	}
	//labels
	m_editElementsIndex[0]->label("El");
	m_editElementsIndex[0]->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editElementsMinFloat[0]->label("Min");
	m_editElementsMinFloat[0]->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editElementsMaxFloat[0]->label("Max");
	m_editElementsMaxFloat[0]->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editElementsMinBut[0]->label("Min");
	m_editElementsMinBut[0]->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editElementsMaxBut[0]->label("Max");
	m_editElementsMaxBut[0]->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_editParameterGroup->hide();


	//LOOPS TAB
	m_loopsTabButton = new Fl_Group(paramPosX, m_execGrilleGroup->y()+m_execGrilleGroup->h()-25, 150, 50, "");	
	m_loopsTabButton->box(FL_RFLAT_BOX);
	m_loopsTabButton->color(FL_INACTIVE_COLOR);
	m_loopsTabButton->end();
	m_loopsTabButton->callback(statTabs,this);
	m_execGrilleGroup->add(m_loopsTabButton);
	m_execCurrentLoopPos = new Fl_Dial(paramPosX+30, m_execGrilleGroup->y()+m_execGrilleGroup->h()-20, 20, 20, "");
	m_execCurrentLoopPos->type(FL_FILL_DIAL);
	m_execCurrentLoopPos->angle1(-90);
	m_execCurrentLoopPos->angle2(270);
	m_execCurrentLoopPos->color(FL_WHITE);
	m_execCurrentLoopPos->color2(FL_SELECTION_COLOR);
	m_execCurrentLoopPos->clear_visible_focus();
	m_loopsTabButton->add(m_execCurrentLoopPos);
	m_execCurrentLoopButton = new TardiButton(m_execCurrentLoopPos->x()+m_execCurrentLoopPos->w()+WIDGETS_SEP, m_execGrilleGroup->y()+m_execGrilleGroup->h()-20, 50, 20, "LOOP");
	m_execCurrentLoopButton->box(FL_FLAT_BOX);
	m_execCurrentLoopButton->down_box(FL_FLAT_BOX);
	m_execCurrentLoopButton->selection_color(FL_SELECTION_COLOR);
	m_execCurrentLoopButton->color(FL_FOREGROUND_COLOR);
	m_execCurrentLoopButton->clear_visible_focus();
	m_execCurrentLoopButton->callback(statExecLoopButton,this);
	m_loopsTabButton->add(m_execCurrentLoopButton);
	m_execCurrentLoopEraseButton = new TardiButton(m_execCurrentLoopButton->x()+m_execCurrentLoopButton->w()+WIDGETS_SEP, m_execGrilleGroup->y()+m_execGrilleGroup->h()-15, 10, 15, "X");
	m_execCurrentLoopEraseButton->box(FL_NO_BOX);
	m_execCurrentLoopEraseButton->labelcolor(FL_RED);
	m_execCurrentLoopEraseButton->clear_visible_focus();
	m_execCurrentLoopEraseButton->callback(statExecLoopButton,this);
	m_loopsTabButton->add(m_execCurrentLoopEraseButton);
	m_loopsTab=new Fl_Group(0,m_monitor->y()+m_monitor->h()+20,this->w(),160,"");
	m_loopsTab->end();
	m_loopsTab->align(FL_ALIGN_TOP|FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
	m_loopsTab->labelcolor(FL_BACKGROUND2_COLOR);
	m_loopsTab->color(FL_INACTIVE_COLOR);
	m_loopsTab->box(FL_FLAT_BOX);
	this->add(m_loopsTab);

	//create loop widgets
	m_loopsTab->begin();
	int loopWidth=m_loopsTab->w()/m_vj->getNbLoops()-WIDGETS_SEP;
	for(unsigned int i=0;i<m_vj->getNbLoops();++i) {
		m_execLoops.push_back(new GrilleGroup(WIDGETS_SEP+m_loopsTab->x()+i*(loopWidth+WIDGETS_SEP), m_loopsTab->y()+WIDGETS_SEP, loopWidth, m_loopsTab->h()-8*WIDGETS_SEP,"", LOOP_GROUP, i, this));
		m_execLoops[i]->callback(statExecLoops,this);
	}
	m_loopsTab->end();

	//INPUTS TAB
	m_inputsTab  = new Fl_Group(m_loopsTabButton->x()+m_loopsTabButton->w(), m_loopsTabButton->y(), 120, 50,"");
	m_inputsTab->color(FL_INACTIVE_COLOR);
	m_inputsTab->box(FL_RFLAT_BOX);
	this->add(m_inputsTab);
	m_inputsTabButton  = new Fl_Button(m_inputsTab->x()+30, m_inputsTab->y(), 50, 25,"INPUTS");
	m_inputsTabButton->color(FL_INACTIVE_COLOR);
	m_inputsTabButton->selection_color(FL_INACTIVE_COLOR);
	m_inputsTabButton->box(FL_NO_BOX);
	m_inputsTabButton->callback(statTabs,this);
	m_inputsTabButton->clear_visible_focus();
	m_inputsTab->add(m_inputsTabButton);
	m_setAddInput = new Fl_Button(m_inputsTabButton->x()+m_inputsTabButton->w()+WIDGETS_SEP, m_inputsTabButton->y()+5, 15, 15,"+");
	m_setAddInput->box(FL_BORDER_BOX);
	m_setAddInput->selection_color(FL_SELECTION_COLOR);
	m_setAddInput->clear_visible_focus();
	m_setAddInput->callback(statSetInputButtons,this);
	m_inputsTab->add(m_setAddInput);
	m_setInputGroup = new Fl_Scroll(WIDGETS_SEP, m_loopsTab->y(), this->w()-2*WIDGETS_SEP, 120, "");
	m_setInputGroup->align(FL_ALIGN_TOP|FL_ALIGN_LEFT);
	m_setInputGroup->box(FL_ROUNDED_BOX);
	m_setInputGroup->color(FL_BACKGROUND2_COLOR);
	m_setInputGroup->type(Fl_Scroll::VERTICAL_ALWAYS);
	this->add(m_setInputGroup);
	m_setInputGroup->hide();
	m_setAddInput->hide();


	//RENDER WINDOW
	m_renderWindow = new RenderWindow(800,600,"VjPirate | Render");	
	m_renderWindow->setResizable(true);
	
	////RENDER  BUTTON
	m_renderButton = new Fl_Toggle_Button(w/2-40,WIDGETS_SEP, 80, 20,"Render");
	m_renderButton->box(FL_FLAT_BOX);
	m_renderButton->selection_color(FL_SELECTION_COLOR);
	m_renderButton->color(FL_BACKGROUND2_COLOR);
	m_renderButton->clear_visible_focus();
	m_renderButton->callback(statRenderButton,this);
	this->add(m_renderButton);
	
	//add these group at the end so that they are on top of other widgets
	this->add(m_editProportionsGroup);
	this->add(m_editParameterGroup);


	//set the idle function
	Fl::add_idle(idle, this);

}


void ExecWindow::idle(void* pnt) {
	ExecWindow *win = (ExecWindow*)pnt;

	//reset all grille and loops 
	Vjpirate* vjp = Vjpirate::getInstance();
    vjp->update();
	vjp->resetGrillesGlCoords();
	
	
	if(win->m_renderWindow->shown()){
		win->m_renderWindow->redraw();
	}
	win->m_monitor->redraw();
	win->redrawLoops();
	win->updateWidgets();

	for(unsigned int i=0;i<win->m_execGrilles.size();++i) {
		if(win->m_execGrilles[i]->getSelected() || win->m_refreshAllPreviews) {
			win->m_execGrilles[i]->redrawPreview();
		}
	}


	if(win->m_refreshAllPreviews) {
		win->m_refreshAllPreviews=false;
	}

	usleep(1000);
	
}



ExecWindow::~ExecWindow()
{
}


int ExecWindow::handle(int event)
{
	if(!Fl_Window::handle(event) && !m_menu->handle(FL_SHORTCUT)) {
		if(event==FL_KEYDOWN) { 
			InputEvent& event = m_vj->getInputEvent(INPUT_KEYBOARD);
			event.key=Fl::event_key();
			event.value=1;
			m_vj->processInputEvent(INPUT_KEYBOARD);
			return 1;
		}
		else if(event==FL_KEYUP) {
			InputEvent& event = m_vj->getInputEvent(INPUT_KEYBOARD);
			event.key=Fl::event_key();
			event.value=0;
			m_vj->processInputEvent(INPUT_KEYBOARD);
			return 1;
		}
	}

	return 0;
}


void ExecWindow::cbWindow(Fl_Widget*)
{
	if (Fl::event()==FL_SHORTCUT) {
		 if(Fl::event_key()==FL_Escape) {
			return; // ignore Escape
		}
	}
	
	cbQuit(this);
}

void ExecWindow::cbTabs(Fl_Widget* w)
{
	m_loopsTab->hide();	
	m_execCurrentLoopEraseButton->hide();
	m_execCurrentLoopPos->hide();
	m_execCurrentLoopButton->color(FL_INACTIVE_COLOR);
	m_execCurrentLoopButton->selection_color(FL_INACTIVE_COLOR);
	for(unsigned int i=0;i<m_execLoops.size();++i) {
		m_execLoops[i]->hide();
	}
	m_setInputGroup->show();	
	m_setAddInput->show();
}

void ExecWindow::cbRenderButton(Fl_Widget*)
{
	if(m_renderButton->value()==0) {
		m_renderWindow->hide();
	}
	else {
		m_renderWindow->show();
	}
}

void ExecWindow::resetInputsTab(bool wait)
{
	if(wait) {
		m_resetInputsTabAsked=true;
	}
	else {
		//if number of inputs has changed
		if(m_setInputs.size()!=m_vj->getNbInputs()) {
			//clear the previous inputs	
			m_setInputGroup->clear();
			m_setInputs.clear();
			//add all the inputs
			for(unsigned int i=0;i<m_vj->getNbInputs();++i) {
				InputGroup* inp=new InputGroup(m_setInputGroup->x()+WIDGETS_SEP, m_setInputGroup->y()+2*WIDGETS_SEP+25*i, m_setInputGroup->w()-20, 20);
				inp->setInputID(i);
				inp->setInput(m_vj->getInput(i));
				m_setInputs.push_back(inp);
				m_setInputGroup->add(inp);
			}
		}
		else {
			//set the input values
			for(unsigned int i=0;i<m_vj->getNbInputs();++i) {
				m_setInputs[i]->setInput(m_vj->getInput(i));
			}
		}

		for(unsigned int i=0;i<m_vj->getNbInputs();++i) {
			m_setInputs[i]->redraw();
		}
		this->redraw();
	}

}



void ExecWindow::cbAddGrille(Fl_Widget* w)
{
	m_vj->addGrille();
	setSelectedExecGrille(m_vj->getCurrentGrilleID());
	m_fileSaved=false;
}

void ExecWindow::cbSetInputButtons(Fl_Widget* w)
{
	if(m_setAddInput->contains(w)) {
		m_vj->addDefaultInput();
		//m_setInputGroup->position(m_setInputGroup->xposition(), (m_setInputs.size()-1)*35+m_setInputGroup->yposition());
	}
}

void ExecWindow::cbEditProportions(Fl_Widget* w)
{
	Grille &selectedGrille = m_vj->getCurrentGrille();

	if(m_editRows->contains(w)) {
		selectedGrille.setParameterInitValue(PARAM_ROWS,m_editRows->value()/Grille::MAX_ROWS);
	}
	else if(m_editCols->contains(w)) {
		selectedGrille.setParameterInitValue(PARAM_COLS,m_editCols->value()/Grille::MAX_COLS);
	}
	else if(m_editHomothety->contains(w)) {
		selectedGrille.setHomothety(m_editHomothety->value());
	}

	//redrawDisplays();
	m_fileSaved=false;
}

void ExecWindow::cbEditOrigin(Fl_Widget* w)
{
	Grille &selectedGrille = m_vj->getCurrentGrille();

	for(unsigned int i=0;i<4;++i) {
		if(m_editOriginButtons[i]->contains(w)) {
			selectedGrille.setOrigin(GRILLE_ORIGIN(i));
		}
	}
	//redrawDisplays();
	m_fileSaved=false;
}

void ExecWindow::cbEditElements(Fl_Widget* w)
{
	Grille &selectedGrille = m_vj->getCurrentGrille();
	Parameter& selectedParam = selectedGrille.getParameter(VJ_PARAM(m_editingParameter));
	if(VJ_PARAM(m_editingParameter)==PARAM_IMAGE) {
		for(unsigned int i=0;i<26;++i) {
			if(m_editElementsMinFloat[i]->contains(w)) {
				selectedParam.setElementRange(i,m_editElementsMinFloat[i]->value(),m_editElementsMaxFloat[i]->value());
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsMaxFloat[i]->contains(w)) {
				selectedParam.setElementRange(i,m_editElementsMinFloat[i]->value(),m_editElementsMaxFloat[i]->value());
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsImBut[i]->contains(w)) {
				//open dialog for directory
				TardiFileBrowser *tFB = new TardiFileBrowser();
				std::string imDir="";
				if(selectedParam.getElement(i).getAnim()) {
					imDir=selectedParam.getElement(i).getAnim()->getDir();
				}
				
				tFB->open("Choose an images directory","dir",imDir);
				while(tFB->shown()) {
					Fl::wait();
				}
				std::string anim=tFB->getResult();
				if(anim.compare("")!=0) {
					Anim* resAnim = m_vj->openAnim(anim);
					//set anim
					selectedParam.setElementAnim(i,resAnim);
					//if anim exist, set max image value
					if(resAnim) {
						m_editElementsMaxFloat[i]->value(resAnim->getNbImagesStr()-1);
						cbEditElements(m_editElementsMaxFloat[i]);
					}
				}
				delete tFB;
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsDup[i]->contains(w)) {
				selectedParam.addElement(i);
			}
			else if(m_editElementsDel[i]->contains(w)) {
				//if element has an anim, remove it, else remove the element
				if(selectedParam.getElement(i).getAnim()!=NULL) {
					selectedParam.getElement(i).setAnim(NULL);
				}
				else {
					selectedParam.removeElement(i);
				}
			}
		}
	}
	else if(VJ_PARAM(m_editingParameter)==PARAM_COLOR) {
		for(unsigned int i=0;i<26;++i) {
			if(m_editElementsMinBut[i]->contains(w)) {
				Element& selectedElement = selectedParam.getElement(i);
				std::string title = "Choose the mininum color";
				fl_color_chooser(title.c_str(),(uchar&)(selectedElement.getColorMinValue(0)),(uchar&)(selectedElement.getColorMinValue(1)),(uchar&)(selectedElement.getColorMinValue(2)));
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsMaxBut[i]->contains(w)) {
				Element& selectedElement = selectedParam.getElement(i);
				std::string title = "Choose the maximum color";
				fl_color_chooser(title.c_str(),(uchar&)(selectedElement.getColorMaxValue(0)),(uchar&)(selectedElement.getColorMaxValue(1)),(uchar&)(selectedElement.getColorMaxValue(2)));
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsDup[i]->contains(w)) {
				selectedParam.addElement(i);
			}
			else if(m_editElementsDel[i]->contains(w)) {
				selectedParam.removeElement(i);
			}
		}

	}
	else {
		for(unsigned int i=0;i<26;++i) {
			if(m_editElementsMinFloat[i]->contains(w)) {
				Element& selectedElement = selectedParam.getElement(i);
				selectedElement.setRange(m_editElementsMinFloat[i]->value(), m_editElementsMaxFloat[i]->value());
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsMaxFloat[i]->contains(w)) {
				Element& selectedElement = selectedParam.getElement(i);
				selectedElement.setRange(m_editElementsMinFloat[i]->value(), m_editElementsMaxFloat[i]->value());
				selectedParam.setValue(selectedParam.getValue());
			}
			else if(m_editElementsDup[i]->contains(w)) {
				selectedParam.addElement(i);
			}
			else if(m_editElementsDel[i]->contains(w)) {
				selectedParam.removeElement(i);
			}
		}
	}
	m_fileSaved=false;
	
	cbParamNames(m_execParamNames[m_editingParameter]);
}



void ExecWindow::cbEditPattern(Fl_Widget* w)
{
	Grille &selectedGrille = m_vj->getCurrentGrille();
	Parameter& selectedParam = selectedGrille.getParameter(VJ_PARAM(m_editingParameter));
	std::string pattern = m_editPattern->value();
	selectedParam.setInitPattern(pattern);	
	m_editPattern->value(selectedParam.getInitPatternStr().c_str());
	m_fileSaved=false;
}

void ExecWindow::cbEditInitValue(Fl_Widget* w)
{
	Grille &selectedGrille = m_vj->getCurrentGrille();
	Parameter& selectedParam = selectedGrille.getParameter(VJ_PARAM(m_editingParameter));
	if(m_editInitValue->contains(w)) {
		selectedParam.setInitValue(m_editInitValue->value());
	}
	m_fileSaved=false;
}


void ExecWindow::cbExecParamValueSliders(Fl_Widget* w)
{
	for(unsigned int i=0;i<PARAMS_NB;++i) {
		if(m_execParamSliders[i]->contains(w)) {
			if(Fl::event_button()==FL_RIGHT_MOUSE) {
				std::vector<int> args(4,0);
				args[0]=Vjpirate::LEARN_INPUT_CREATE;
				args[1]=GRILLE_INPUT;
				args[2]=m_vj->getCurrentGrilleID()+1;
				args[3]=i;
				openLearnMenu(args);
			}
			else {
				m_vj->setCurrentGrilleParameterValue((VJ_PARAM)i,((Fl_Slider*)w)->value());
			}
		}
	}
}

void ExecWindow::cbParamNames(Fl_Widget* w)
{
	unsigned int index=0;
	//check rows and columns buttons
	for(unsigned int i=PARAM_COLS;i<=PARAM_ROWS;++i) {
		if(m_execParamNames[i]->contains(w)) {
			index=i;
			if(m_execParamNames[i]->value()==0) {
				if(!m_setInputGroup->visible()) {
					for(unsigned int l=0;l<m_execLoops.size();++l) {
						m_execLoops[l]->show();
					}
				}
				m_editProportionsGroup->hide();	
			}
			else {
				m_editingParameter=index;
				if(!m_setInputGroup->visible()) {
					for(unsigned int l=0;l<m_execLoops.size();++l) {
						m_execLoops[l]->show();
					}
				}
				m_editProportionsGroup->hide();	
				m_editParameterGroup->hide();	
				m_editProportionsGroup->position(m_execGrilleGroup->x()+WIDGETS_SEP*3,m_execParamNames[PARAM_COLS]->y()+2*EXEC_PARAMS_H);
				m_editProportionsGroup->show();	
			}
		}
	}

	//check the other params buttons
	for(unsigned int i=PARAM_SIZE;i<PARAMS_NB;++i) {
		if(m_execParamNames[i]->contains(w)) {
			index=i;
			if(m_execParamNames[i]->value()==0) {
				if(!m_setInputGroup->visible()) {
					for(unsigned int l=0;l<m_execLoops.size();++l) {
						m_execLoops[l]->show();
					}
				}
				m_execCurrentLoopPos->show();
				m_editParameterGroup->hide();	
			}
			else {
				m_editingParameter=index;
				//get parameter values
				Grille& selectedGrille = m_vj->getCurrentGrille();
				Parameter& selectedParam = selectedGrille.getParameter(VJ_PARAM(index));
				//get pattern
				m_editPattern->value(selectedParam.getInitPatternStr().c_str());
				//clearn elements
				for(unsigned int e=0;e<ALPHA.size();++e) {
					m_editElementsIndex[e]->hide();
					m_editElementsDup[e]->hide();
					m_editElementsDel[e]->hide();
					m_editElementsMinFloat[e]->hide();
					m_editElementsMaxFloat[e]->hide();
					m_editElementsMinBut[e]->hide();
					m_editElementsMaxBut[e]->hide();
					m_editElementsImBut[e]->hide();
				}
				//get elements
				for(unsigned int e=0;e<selectedParam.getNbElements();++e) {
					Element& el = selectedParam.getElement(e);
					m_editElementsIndex[e]->show();
					m_editElementsDup[e]->show();
					m_editElementsDel[e]->show();
					switch(VJ_PARAM(index)) {
						case PARAM_COLOR: {
							m_editElementsMinBut[e]->show();
							m_editElementsMaxBut[e]->show();

							m_editElementsMinBut[e]->color(fl_rgb_color(el.getColorMinValue(0),el.getColorMinValue(1),el.getColorMinValue(2)));
							m_editElementsMaxBut[e]->color(fl_rgb_color(el.getColorMaxValue(0),el.getColorMaxValue(1),el.getColorMaxValue(2)));

						}break;
						case PARAM_IMAGE: {
							m_editElementsMinFloat[e]->show();
							m_editElementsMaxFloat[e]->show();
							m_editElementsImBut[e]->show();

							m_editElementsMinFloat[e]->value(el.getMinValue());
							m_editElementsMaxFloat[e]->value(el.getMaxValue());
							if(el.getAnim()) {
								m_editElementsImBut[e]->copy_label(el.getAnim()->getShortDir().c_str());
							}
							else {
								m_editElementsImBut[e]->copy_label("no images");
							}

						}break;
						default: {
							m_editElementsMinFloat[e]->show();
							m_editElementsMaxFloat[e]->show();

							m_editElementsMinFloat[e]->value(el.getMinValue());
							m_editElementsMaxFloat[e]->value(el.getMaxValue());
						}
					}	
				}

				//get init value
				m_editInitValue->value(selectedParam.getInitValue());

				//display
				//m_loopsTab->hide();	
				for(unsigned int l=0;l<m_execLoops.size();++l) {
					m_execLoops[l]->hide();
				}
				m_execCurrentLoopPos->hide();
				m_editProportionsGroup->hide();	
				m_editParameterGroup->hide();	
				m_editParameterGroup->position(m_execGrilleGroup->x()+WIDGETS_SEP*3,m_execParamNames[i]->y()+2*EXEC_PARAMS_H);
				m_editParameterGroup->show();	
			}
		}
	}
	
	for(unsigned int i=0;i<PARAMS_NB;++i) {
		if(i!=index) {
			m_execParamNames[i]->value(0);
		}
	}
}

void ExecWindow::cbExecParamPermut(Fl_Widget* w)
{
	for(unsigned int i=2;i<PARAMS_NB;++i) {
		if(m_execParamPermut[i-2]->contains(w)) {
			if(Fl::event_button()==FL_RIGHT_MOUSE) {
				std::vector<int> args(4,0);
				args[0]=Vjpirate::LEARN_INPUT_CREATE;
				args[1]=GRILLE_INPUT;
				args[2]=m_vj->getCurrentGrilleID()+1;
				args[3]=i-2+PARAMS_NB;
				openLearnMenu(args);
			}
			else {
				m_vj->setCurrentGrilleParameterPatternPermutation((VJ_PARAM)i,((Fl_Slider*)w)->value());
				m_execParamPermut[i-2]->copy_label(m_vj->getCurrentGrille().getParameter(VJ_PARAM(i)).getPatternStr().c_str());
			}
		}
	}
}

void ExecWindow::setSelectedExecGrille(const unsigned int& gri, const bool& wait)
{
	if(wait) {
		m_selectedExecGrilleChanged=true;
		m_selectedExecGrilleValue=gri;
	}
	else {
		if(gri<m_vj->getNbGrilles()) {
			m_execGrilleScroll->selectGrille(gri);

			setCurrentGrilleAct(m_vj->getCurrentGrille().getActive());
			int selLoop = Vjpirate::SELECTED_LOOP;
			setLoopState(selLoop, m_vj->getCurrentGrille().getLoopState());
			m_execBlendButton->value(m_vj->getCurrentGrille().getBlend());
			m_execAudioButton->value(m_vj->getCurrentGrille().getAudioActive());

			m_execParamSliders[PARAM_COLS]->value(m_vj->getCurrentGrille().getParameter(PARAM_COLS).getFirstElementValue()/Grille::MAX_COLS);
			m_execParamSliders[PARAM_ROWS]->value(m_vj->getCurrentGrille().getParameter(PARAM_ROWS).getFirstElementValue()/Grille::MAX_ROWS);


			for(unsigned int i=0;i<PARAMS_NB;++i) {
				//display the audio input
				if(m_vj->isParameterofCurrentGrilleControlledByAudio(i)) {
					m_execParamAudio[i]->show();
					if(m_vj->getCurrentGrille().getAudioActive()) {
						m_execParamSliders[i]->deactivate();
						m_execParamSliders[i]->color2(FL_BACKGROUND2_COLOR);
						m_execParamAudio[i]->value(1);
						m_execParamAudio[i]->labelcolor(FL_FOREGROUND_COLOR);
						m_execParamAudio[i]->redraw_label();
					}
					else {
						m_execParamSliders[i]->activate();
						m_execParamSliders[i]->color2(FL_SELECTION_COLOR);
						m_execParamAudio[i]->value(0);
						m_execParamAudio[i]->labelcolor(FL_BACKGROUND2_COLOR);
						m_execParamAudio[i]->redraw_label();
					}
				}
				else {
					m_execParamSliders[i]->activate();
					m_execParamSliders[i]->color2(FL_SELECTION_COLOR);
					m_execParamAudio[i]->hide();
				}

				//if not columns nor rows
				if(i>=2) {
					//set the value of the slider
					m_execParamSliders[i]->value(m_vj->getCurrentGrille().getParameter(VJ_PARAM(i)).getValue());

					//set the value of the permut
					m_execParamPermut[i-2]->copy_label(m_vj->getCurrentGrille().getParameter(VJ_PARAM(i)).getPatternStr().c_str());
					m_execParamPermut[i-2]->value(m_vj->getCurrentGrille().getParameter(VJ_PARAM(i)).getPatternPermutation());
				
					//display the audio input for permut
					if(m_vj->isParameterPermutofCurrentGrilleControlledByAudio(i)) {
						m_execParamPermutAudio[i-2]->show();
						if(m_vj->getCurrentGrille().getAudioActive()) {
							m_execParamPermut[i-2]->deactivate();
							m_execParamPermutAudio[i-2]->value(1);
							m_execParamPermutAudio[i-2]->labelcolor(FL_FOREGROUND_COLOR);
							m_execParamPermutAudio[i-2]->redraw_label();
						}
						else {
							m_execParamPermut[i-2]->activate();
							m_execParamPermutAudio[i-2]->value(0);
							m_execParamPermutAudio[i-2]->labelcolor(FL_BACKGROUND2_COLOR);
							m_execParamPermutAudio[i-2]->redraw_label();
						}
					}
					else {
						m_execParamPermut[i-2]->activate();
						m_execParamPermutAudio[i-2]->hide();
					}
				}
			}

			//fill the edit group for proportions
			m_editRows->value(m_vj->getCurrentGrille().getParameter(PARAM_ROWS).getInitValue()*Grille::MAX_ROWS);
			m_editCols->value(m_vj->getCurrentGrille().getParameter(PARAM_COLS).getInitValue()*Grille::MAX_COLS);
			m_editHomothety->value(m_vj->getCurrentGrille().getHomothety());
			unsigned int origin = m_vj->getCurrentGrille().getOrigin();
			for(unsigned int i=0;i<4;++i) {
				if(i==origin) {
					m_editOriginButtons[0]->value(1);
				}
				else {
					m_editOriginButtons[0]->value(0);
				}
			}
		}
	}
}


void ExecWindow::cbExecGrillesScroll(Fl_Widget* w)
{
	std::vector<int> args(4,0);
	args[0]=Vjpirate::LEARN_INPUT_CREATE;
	args[1]=APPLICATION_INPUT;
	args[2]=Vjpirate::SELECTED_GRILLE;
	args[3]=PARAM_SELECT;
	openLearnMenu(args);
}

void ExecWindow::cbExecGrilles(Fl_Widget* w)
{
	for(unsigned int i=0;i<m_execGrilles.size();++i) {
		if(m_execGrilles[i]->contains(w)) {
			m_vj->setCurrentGrille(i);
		}
	}
}

void ExecWindow::cbExecSelectAct(Fl_Widget* w)
{
	if(Fl::event_button()==FL_RIGHT_MOUSE) {
		std::vector<int> args(4,0);
		args[0]=Vjpirate::LEARN_INPUT_CREATE;
		args[1]=APPLICATION_INPUT;
		args[2]=Vjpirate::SELECTED_GRILLE;
		args[3]=PARAM_ACTIVE_SELECT;
		openLearnMenu(args);
	}
	else {
		m_vj->setSelectActive(m_execSelectActButton->value());	
	}
}

void ExecWindow::cbExecAudioButton(Fl_Widget* w)
{
	if(Fl::event_button()==FL_RIGHT_MOUSE) {
		std::vector<int> args(4,0);
		args[0]=Vjpirate::LEARN_INPUT_CREATE;
		args[1]=GRILLE_INPUT;
		args[2]=Vjpirate::SELECTED_GRILLE;
		args[3]=PARAM_AUDIO+PARAMS_NB+PERMUT_PARAMS_NB;
		openLearnMenu(args);
	}
	else {
		Grille& selectedGrille = m_vj->getCurrentGrille();
		selectedGrille.setAudioActive(m_execAudioButton->value());
		m_vj->resetAudioInputStoredValues(GRILLE_INPUT, 
											m_vj->getCurrentGrilleID());
		setSelectedExecGrille(m_vj->getCurrentGrilleID());
	}
}

void ExecWindow::cbExecBlendButton(Fl_Widget* w)
{
	if(Fl::event_button()==FL_RIGHT_MOUSE) {
		std::vector<int> args(4,0);
		args[0]=Vjpirate::LEARN_INPUT_CREATE;
		args[1]=GRILLE_INPUT;
		args[2]=Vjpirate::SELECTED_GRILLE;
		args[3]=PARAM_BLEND+PARAMS_NB+PERMUT_PARAMS_NB;
		openLearnMenu(args);
	}
	else {
		Grille& selectedGrille = m_vj->getCurrentGrille();
		selectedGrille.setBlend(m_execBlendButton->value());
	}
}

void ExecWindow::cbExecInitValuesButton(Fl_Widget* w)
{
	if(m_execInitValuesButton->contains(w)) {
		if(Fl::event_button()==FL_RIGHT_MOUSE) {
			std::vector<int> args(4,0);
			args[0]=Vjpirate::LEARN_INPUT_CREATE;
			args[1]=GRILLE_INPUT;
			args[2]=Vjpirate::SELECTED_GRILLE;
			args[3]=PARAM_REINIT+PARAMS_NB+PERMUT_PARAMS_NB;
			openLearnMenu(args);
		}
		else {
			Grille& selectedGrille = m_vj->getCurrentGrille();
			selectedGrille.reinit();
			setSelectedExecGrille(m_vj->getCurrentGrilleID());
		}
	}
	else if(m_editGrabInitValue->contains(w)) {
		Grille& selectedGrille = m_vj->getCurrentGrille();
		selectedGrille.grabInitValues();
		setSelectedExecGrille(m_vj->getCurrentGrilleID());
	}
}

void ExecWindow::cbExecActButton(Fl_Widget* w)
{
	if(Fl::event_button()==FL_RIGHT_MOUSE) {
		std::vector<int> args(4,0);
		args[0]=Vjpirate::LEARN_INPUT_CREATE;
		args[1]=GRILLE_INPUT;
		args[2]=Vjpirate::SELECTED_GRILLE;
		args[3]=PARAM_ACTIVATION+PARAMS_NB+PERMUT_PARAMS_NB;
		openLearnMenu(args);
	}
	else {
		Grille& selectedGrille = m_vj->getCurrentGrille();
		selectedGrille.setActive(m_execActButton->value());
		m_vj->resetAudioInputStoredValues(GRILLE_INPUT, 
											m_vj->getCurrentGrilleID());
	}
}

void ExecWindow::openLearnMenu(std::vector<int> args)
{
	m_learnArgs=args;
	m_learnMenu->popup();	
}

void ExecWindow::cbLearnMenu(Fl_Widget* w)
{
	if(m_learnMenu->value()==0) {
		m_vj->learnInput(true, m_learnArgs);
	}
	else {
		Input inp;
		inp.group=INPUT_GROUP(m_learnArgs[1]);
		inp.grille=m_learnArgs[2];
		inp.parameter=m_learnArgs[3];
		inp.type=INPUT_AUDIO;
		inp.audioFeature=ANALYSIS_RMS;
		inp.mode=MODE_NORMAL;
		m_vj->addInput(inp);
	}
						
}


void ExecWindow::cbExecLoopButton(Fl_Widget* w)
{
	if(m_execCurrentLoopButton->contains(w)) {
		//if currently on the loop tab
		if(m_execCurrentLoopPos->visible()) {
			if(Fl::event_button()==FL_RIGHT_MOUSE) {
				std::vector<int> args(4,0);
				args[0]=Vjpirate::LEARN_INPUT_CREATE;
				args[1]=APPLICATION_INPUT;
				args[2]=Vjpirate::SELECTED_GRILLE;
				args[3]=PARAM_LOOP_RECORD;
				openLearnMenu(args);
			}
			else {
				int selLoop=Vjpirate::SELECTED_LOOP;
				m_vj->triggerLoop(LOOP_ACTION_TRIGGER, selLoop);
			}
		}
		else {
			m_setInputGroup->hide();	
			m_setAddInput->hide();	
			m_execCurrentLoopEraseButton->show();
			m_execCurrentLoopButton->color(FL_BACKGROUND2_COLOR);
			m_execCurrentLoopButton->selection_color(FL_SELECTION_COLOR);
			m_execCurrentLoopPos->show();
			for(unsigned int i=0;i<m_execLoops.size();++i) {
				m_execLoops[i]->show();
			}
			m_loopsTab->show();	
		}
	}
	else if(m_execCurrentLoopEraseButton->contains(w)) {
		if(Fl::event_button()==FL_RIGHT_MOUSE) {
			std::vector<int> args(4,0);
			args[0]=Vjpirate::LEARN_INPUT_CREATE;
			args[1]=APPLICATION_INPUT;
			args[2]=Vjpirate::SELECTED_GRILLE;
			args[3]=PARAM_LOOP_ERASE;
			openLearnMenu(args);
		}
		else {
			int selLoop=Vjpirate::SELECTED_LOOP;
			m_vj->eraseLoop(selLoop);
		}
	}
}

void ExecWindow::cbExecLoops(Fl_Widget* w)
{
	for(unsigned int i=0;i<m_execLoops.size();++i) {
		if(m_execLoops[i]->contains(w)) {
			if(Fl::event_button()==FL_LEFT_MOUSE) {
				m_vj->triggerLoop(LOOP_ACTION_TRIGGER, i);
			}
		}
	}
}

void ExecWindow::redrawLoops()
{
	for(unsigned int i=0;i<m_execLoops.size();++i) {
        if(m_execLoops[i]->visible()) {
            m_execLoops[i]->redrawPreview();
        }
	}
}


void ExecWindow::cbExecMaster(Fl_Widget*)
{
	if(Fl::event_button()==FL_RIGHT_MOUSE) {
		std::vector<int> args(4,0);
		args[0]=Vjpirate::LEARN_INPUT_CREATE;
		args[1]=APPLICATION_INPUT;
		args[2]=Vjpirate::SELECTED_GRILLE;
		args[3]=PARAM_MASTER;
		openLearnMenu(args);
	}
	else {
		m_vj->setMasterTransparency((m_execMasterSlider->value()-1.0)*-1.0);
	}
}


void ExecWindow::cbOpen(Fl_Widget* w)
{
	if(!m_fileSaved) {
		switch(fl_choice("File not saved ! Quit without saving ?", "Quit without saving","Save then Quit" ,"Don't quit")) {
			case 0:
			break;
			case 1: {
				cbSave(this);
				if(!m_fileSaved)
					return;
			}
			break;
			default: return;
				
		}
	}	

	TardiFileBrowser *tFB = new TardiFileBrowser();
	std::string currentFileStr="";
	if(m_vj->getFileName().compare("")!=0) {
		char* currentFile= new char[FL_PATH_MAX];
		fl_filename_absolute(currentFile,m_vj->getFileName().c_str());
		currentFileStr = currentFile;
		delete [] currentFile;
	}
	tFB->open("Choose a .vjp file","{*.vjp}",currentFileStr);
	while(tFB->shown())
	{
		Fl::wait();
	}
	std::string newFile=tFB->getResult();
	if(newFile.compare("")!=0)
	{
		m_vj->openFile(newFile.c_str());
	}
	delete tFB;
	resetInputsTab();
}

void ExecWindow::cbSave(Fl_Widget* w)
{
	//if no previous file
	if(m_vj->getFileName().compare("")==0) {
		cbSaveAs(this);
	}
	else {
		char* buffer = new char[m_vj->getFileName().size()+1];
		strcpy(buffer,m_vj->getFileName().c_str());
		m_vj->saveToFile(buffer);
		delete [] buffer;
		m_fileSaved=true;
	}
	
}

void ExecWindow::cbSaveAs(Fl_Widget* w)
{
	//open a dialog
	TardiFileBrowser *tFB = new TardiFileBrowser();
	tFB->open("Save to a .vjp file","{*.vjp}","");
	while(tFB->shown())
	{
		Fl::wait();
	}
	std::string strNewFile=tFB->getResult();
	if(strNewFile.compare("")!=0) {
		if(!fl_filename_match(strNewFile.c_str(),"*.vjp")) {
			strNewFile+=".vjp";
		}
		m_vj->saveToFile(strNewFile.c_str());
		std::string title = "VjPirate: ";
		title+=m_vj->getShortFileName();
		label(title.c_str());
		m_fileSaved=true;
	}
	delete tFB;
}

void ExecWindow::cbQuit(Fl_Widget* w)
{
	if(!m_fileSaved) {
		switch(fl_choice("File not saved ! Quit without saving ?", "Quit without saving","Save then Quit" ,"Don't quit")) {
			case 0:
			break;
			case 1: {
				cbSave(this);
				if(!m_fileSaved)
					return;
			}
			break;
			default: return;
				
		}
	}	

	m_vj->stop();

	exit(0);
}

void ExecWindow::cbImportInputs(Fl_Widget*)
{
	TardiFileBrowser *tFB = new TardiFileBrowser();
	std::string currentFileStr="";
	if(m_vj->getFileName().compare("")!=0) {
		char* currentFile= new char[FL_PATH_MAX];
		fl_filename_absolute(currentFile,m_vj->getFileName().c_str());
		currentFileStr = currentFile;
		delete [] currentFile;
	}
	tFB->open("Choose a .vjp file to import inputs from","{*.vjp}",currentFileStr);
	while(tFB->shown()) {
		Fl::wait();
	}
	std::string newFile=tFB->getResult();
	if(newFile.compare("")!=0) {
		m_vj->importInputs(newFile.c_str());
		m_fileSaved=false;
	}
	delete tFB;
	resetInputsTab();
}

void ExecWindow::setTitle(const std::string& t, const bool& wait)
{
	if(wait) {
		m_setTitleAsked=true;
		m_setTitle=t;
	}
	else {
		copy_label(m_setTitle.c_str());	
	}
}

void ExecWindow::reset(const bool& wait)
{
	
	//EXEC TAB
	//clear everything and get the grilles
	resetExecGrilles(wait);

	if(wait) {
		m_resetAsked=true;
	}
	else {
		//get the loops

		//show everything
		for(unsigned int i=0;i<m_execGrilles.size();++i) {
			m_execGrilles[i]->show();
		}

		for(unsigned int i=0;i<m_execLoops.size();++i) {
			m_execLoops[i]->setLoopStateValue(LOOP_EMPTY);	
			m_execLoops[i]->show();
		}

		m_monitor->show();
		cbExecGrilles(m_execGrilles[0]);
	}

}

void ExecWindow::resetExecGrilles(bool wait)
{
	if(wait){
		m_resetExecGrillesAsked=true;
	}
	else {
		int grillesDiff = m_vj->getNbGrilles() - m_execGrilles.size();
		if(grillesDiff>=0) {
			m_execGrilleScroll->begin();
			for(int i=0;i<grillesDiff;++i) {
				m_execGrilles.push_back(new GrilleGroup(0, 0, ((m_execGrilleScroll->h()-20)/3)*4, m_execGrilleScroll->h()-20, m_vj->getGrille(m_execGrilles.size()).getName().c_str(), GRILLE_GROUP, m_execGrilles.size(), this));
				m_execGrilles[m_execGrilles.size()-1]->callback(statExecGrilles,this);
				m_execGrilleScroll->add(m_execGrilles[m_execGrilles.size()-1]);
			}
			m_execGrilleScroll->end();
		}
		else {
			grillesDiff*=-1;
			for(int i=0;i<grillesDiff;++i) {
				m_execGrilleScroll->remove(m_execGrilles[m_execGrilles.size()-1]);
				delete m_execGrilles[m_execGrilles.size()-1];
				m_execGrilles.pop_back();
			}
		}
		for(unsigned int i=0;i<m_vj->getNbGrilles();++i) {
			m_execGrilles[i]->setLabel(m_vj->getGrille(i).getName());
			m_execGrilles[i]->show();
			m_execGrilles[i]->redraw();
		}
		m_execGrilleScroll->redraw();
	}
	m_refreshAllPreviews=true;
}

void ExecWindow::updateWidgets()
{

	if(m_openFile) {
		m_vj->openFile(m_openFileName.c_str(), false);
		m_openFile=false;
	}

	m_execSelectActButton->updateValue();

	for(unsigned int i=0;i<PARAMS_NB;++i) {
		m_execParamSliders[i]->updateValue();
	}

	for(unsigned int i=0;i<PERMUT_PARAMS_NB;++i) {
		m_execParamPermut[i]->updateValue();
	}

	m_execActButton->updateValue();
	m_execBlendButton->updateValue();
	m_execAudioButton->updateValue();
	m_execCurrentLoopButton->updateColor();

	if(m_execCurrentLoopPosChanged) {
		m_execCurrentLoopPos->value(m_execCurrentLoopPosValue);
		m_execCurrentLoopPosChanged=false;
		m_execCurrentLoopPos->redraw();
	}
	
	for(unsigned int i=0;i<m_execLoops.size();++i) {
		m_execLoops[i]->updateValues();
	}

	if(m_execMasterSliderChanged) {
		setMasterTransparency(m_execMasterSliderValue, false);
		m_execMasterSliderChanged=false;
		m_execMasterSlider->redraw();
	}

	if(m_selectedExecGrilleChanged) {
		setSelectedExecGrille(m_selectedExecGrilleValue, false);
		m_selectedExecGrilleChanged=false;
		this->redraw();
	}

	if(m_resetExecGrillesAsked) {
		resetExecGrilles(false);
		m_resetExecGrillesAsked=false;
		this->redraw();
	}

	if(m_resetAsked) {
		reset(false);
		m_resetAsked=false;
		this->redraw();
	}

	if(m_resetInputsTabAsked) {
		resetInputsTab(false);
		m_resetInputsTabAsked=false;
		m_inputsTab->redraw();
	}

	if(m_reinitGLAsked) {
		reinitGl(false);
		m_reinitGLAsked=false;
		this->redraw();
	}

	if(m_setTitleAsked) {
		setTitle(m_setTitle, false);
		m_setTitleAsked=false;
		this->redraw();
	}

}

void ExecWindow::setMasterTransparency(const float& val, const bool& wait)
{
	if(wait) {
		m_execMasterSliderChanged=true;
		m_execMasterSliderValue=val;
	}else {
		m_execMasterSlider->value(val);

	}
}

void ExecWindow::setLoopState(const int& loop, const LOOP_STATE& state) 
{
	if(loop==Vjpirate::SELECTED_LOOP) {
		switch(state) {
			case LOOP_EMPTY:m_execCurrentLoopButton->setColor(FL_BACKGROUND2_COLOR);break;
			case LOOP_RECORDING:m_execCurrentLoopButton->setColor(FL_RED);break;
			case LOOP_OVERDUBBING:m_execCurrentLoopButton->setColor(FL_RED);break;
			case LOOP_PLAYING:m_execCurrentLoopButton->setColor(FL_SELECTION_COLOR);break;
			case LOOP_STOPPED:m_execCurrentLoopButton->setColor(FL_BACKGROUND2_COLOR);break;
		}
	}
	else {
		m_execLoops[loop]->setLoopStateValue(state);
	}
}

void ExecWindow::setLoopTransparency(const int& loop, const float& val) 
{
	m_execLoops[loop]->setLoopTransparency(val);
}

void ExecWindow::setLoopSpeed(const int& loop, const float& val) 
{
	m_execLoops[loop]->setLoopSpeed(val);
}

void ExecWindow::setLoopPos(const int& loop, const float& pos)
{
	if(loop==Vjpirate::SELECTED_LOOP) {
		m_execCurrentLoopPosValue=pos;
		m_execCurrentLoopPosChanged=true;
	}
	else {
		m_execLoops[loop]->setLoopPos(pos);
	}
}

void ExecWindow::setCurrentGrilleParameterValue(const int& param, const float& val)
{
	m_execParamSliders[param]->setValue(val);
}

void ExecWindow::setCurrentGrilleParameterPatternPermutation(const int& param, const float& val)
{
	m_execParamPermut[param-2]->setValue(val);
	m_execParamPermut[param-2]->setCopyLabel(m_vj->getCurrentGrille().getParameter(VJ_PARAM(param)).getPatternStr());
}

void ExecWindow::setCurrentGrilleAct(const bool& act)
{
	m_execActButton->setValue(act);
}

void ExecWindow::setCurrentGrilleAudio(const bool& act)
{
	m_execAudioButton->setValue(act);
}

void ExecWindow::setCurrentGrilleBlend(const bool& blend)
{
	m_execBlendButton->setValue(blend);
}

