/***************************************************************************
 *            ExecWindow.h
 *
 *  Mon Sep  4 09:56:39 2006
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/gl.h>
#include <FL/Fl_Tabs.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Color_Chooser.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Value_Input.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Counter.H>
#include <FL/Fl_Dial.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Shared_Image.H>
#include <FL/Fl_Hold_Browser.H>
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <locale>
#include <ctype.h>
 
#ifndef _EXECWINDOW_H
#define _EXECWINDOW_H

#include "RenderWindow.hpp"
#include "TardiFileBrowser.hpp"
#include "GrilleGroup.hpp"
#include "GrilleScroll.hpp"
#include "PatternValue.hpp"
#include "TardiSlider.hpp"
#include "TardiButton.hpp"
#include "TardiValueOutput.hpp"
#include "InputGroup.hpp"

enum VJ_COLORS{VJ_BLUE, VJ_DARK_GREY, VJ_GREY, VJ_BLACK};

class Vjpirate;

class ExecWindow : public Fl_Double_Window 
{
  public:
		ExecWindow(int ,int ,const char*,Vjpirate* );
		~ExecWindow();
		void setTitle(const std::string&, const bool& wait=true);
		void reset(const bool& wait=true);
		int handle(int);
		
		void setMasterTransparency(const float& val, const bool& wait=true);
		void setSelectActive(const bool& act){m_execSelectActButton->setValue(act);}

		void setCurrentGrilleParameterValue(const int&, const float&);
		void setCurrentGrilleParameterPatternPermutation(const int&, const float&);
		void setCurrentGrilleAct(const bool&);
		void setCurrentGrilleBlend(const bool&);
		void setCurrentGrilleAudio(const bool&);
		void setSelectedExecGrille(const unsigned int& gri, const bool& wait=true);

		inline void generateTexture(GLuint *id){m_monitor->generateTexture(id);}
		inline void deleteTexture(GLuint *id){m_monitor->deleteTexture(id);}

		static void idle(void* pnt);
		
		inline void reinitGl(bool wait=true){if(wait){m_reinitGLAsked=true;} else{m_monitor->invalidate();}}

		inline void setOpenfile(std::string fileName){m_openFile=true,m_openFileName=fileName;}

		void setLoopState(const int&, const LOOP_STATE&);
		void setLoopTransparency(const int&, const float&);
		void setLoopSpeed(const int&, const float&);
		void setLoopPos(const int&, const float&);
		void redrawLoops();

		static void statWindow(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbWindow(w);}	
		void cbWindow(Fl_Widget*);

		//menu callback
		static void statOpen(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbOpen(w);}	
		void cbOpen(Fl_Widget*);
		static void statSave(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbSave(w);}	
		void cbSave(Fl_Widget*);
		static void statSaveAs(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbSaveAs(w);}	
		void cbSaveAs(Fl_Widget*);
		static void statQuit(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbQuit(w);}	
		void cbQuit(Fl_Widget*);
		static void statImportInputs(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbImportInputs(w);}	
		void cbImportInputs(Fl_Widget*);
		
		//tabs
		static void statTabs(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbTabs(w);}	
		void cbTabs(Fl_Widget*);

		//render 
		static void statRenderButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbRenderButton(w);}	
		void cbRenderButton(Fl_Widget*);


		//inputs tab
		void resetInputsTab(bool wait=true);
		static void statSetInputButtons(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbSetInputButtons(w);}	
		void cbSetInputButtons(Fl_Widget*);

		//creation tab
		static void statEditProportions(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbEditProportions(w);}
		void cbEditProportions(Fl_Widget*);
		static void statEditOrigin(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbEditOrigin(w);}
		void cbEditOrigin(Fl_Widget*);
		static void statEditElements(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbEditElements(w);}
		void cbEditElements(Fl_Widget*);
		static void statEditPattern(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbEditPattern(w);}
		void cbEditPattern(Fl_Widget*);
		static void statEditExternalInput(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbEditExternalInput(w);}
		void cbEditExternalInput(Fl_Widget*);
		static void statEditInitValue(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbEditInitValue(w);}
		void cbEditInitValue(Fl_Widget*);
		static void statAddGrille(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbAddGrille(w);}
		void cbAddGrille(Fl_Widget*);
		


		//exec tab
		static void statExecParamValueSliders(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecParamValueSliders(w);}
		void cbExecParamValueSliders(Fl_Widget*);
		static void statParamNames(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbParamNames(w);}
		void cbParamNames(Fl_Widget*);
		static void statExecParamPermut(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecParamPermut(w);}
		void cbExecParamPermut(Fl_Widget*);
		static void statExecAudioButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecAudioButton(w);}
		void cbExecAudioButton(Fl_Widget*);
		static void statExecSelectAct(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecSelectAct(w);}
		void cbExecSelectAct(Fl_Widget*);
		static void statExecGrilles(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecGrilles(w);}
		void cbExecGrilles(Fl_Widget*);
		static void statExecGrillesScroll(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecGrillesScroll(w);}
		void cbExecGrillesScroll(Fl_Widget*);
		static void statExecEditButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecEditButton(w);}
		void cbExecEditButton(Fl_Widget*);
		static void statExecInitValuesButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecInitValuesButton(w);}
		void cbExecInitValuesButton(Fl_Widget*);
		static void statExecBlendButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecBlendButton(w);}
		void cbExecBlendButton(Fl_Widget*);
		static void statExecActButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecActButton(w);}
		void cbExecActButton(Fl_Widget*);
		static void statExecLoopButton(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecLoopButton(w);}
		void cbExecLoopButton(Fl_Widget*);
		static void statExecLoops(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecLoops(w);}
		void cbExecLoops(Fl_Widget*);
		static void statExecMaster(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbExecMaster(w);}
		void cbExecMaster(Fl_Widget*);
		static void statLearnMenu(Fl_Widget* w,void* f){ExecWindow *tmpf = static_cast<ExecWindow *>(f);tmpf->cbLearnMenu(w);}
		void cbLearnMenu(Fl_Widget*);
		void openLearnMenu(std::vector<int>);
		void resetExecGrilles(bool wait=true);

		void updateWidgets();

  private:
		Vjpirate *m_vj;	

		bool m_fileSaved;

		Fl_Group *m_windowBar;
		Fl_Menu_Bar *m_menu;

		Fl_Menu_Button *m_learnMenu;
		std::vector<int> m_learnArgs;

		//Render window
		RenderWindow *m_renderWindow;
		Fl_Toggle_Button *m_renderButton;

		//inputs tab
		Fl_Group *m_inputsTab;
		Fl_Button *m_inputsTabButton;
		Fl_Scroll *m_setInputGroup;
		Fl_Button *m_setAddInput;
		std::vector<InputGroup*> m_setInputs;

		Fl_Slider* m_execMasterSlider;
		RenderWindow *m_monitor;
		Fl_Box* m_execSeparator;
	
		Fl_Group *m_execGrilleGroup;
		GrilleScroll *m_execGrilleScroll;
		std::vector<GrilleGroup*> m_execGrilles;
		TardiButton* m_execSelectActButton;
		Fl_Button* m_addGrille;
		Fl_Toggle_Button* m_execParamNames[PARAMS_NB];
		TardiSlider* m_execParamSliders[PARAMS_NB];
		PatternValue* m_execParamPermut[PERMUT_PARAMS_NB];
		Fl_Button* m_execParamAudio[PARAMS_NB];
		Fl_Button* m_execParamPermutAudio[PERMUT_PARAMS_NB];
		TardiButton* m_execInitValuesButton;
		TardiButton* m_execAudioButton;
		TardiButton* m_execBlendButton;
		TardiButton* m_execActButton;
		TardiButton* m_execCurrentLoopButton;

		//edit groups
		Fl_Group *m_editProportionsGroup;
		Fl_Group *m_editOrigin;
			Fl_Check_Button *m_editOriginButtons[4];
		Fl_Check_Button *m_editHomothety;
		Fl_Value_Input *m_editRows;
		Fl_Value_Input *m_editCols;

		Fl_Group *m_editParameterGroup;
		int m_editingParameter;
		Fl_Group *m_editPatterns;
			Fl_Input *m_editPattern;
			Fl_Value_Input *m_editInitValue;
			Fl_Button *m_editGrabInitValue;
			Fl_Button *m_editGrabAllInitValues;
			Fl_Scroll *m_editElements;
				Fl_Value_Input *m_editElementsMinFloat[26];
				Fl_Value_Input *m_editElementsMaxFloat[26];
				Fl_Button* m_editElementsMinBut[26];
				Fl_Button* m_editElementsMaxBut[26];
				Fl_Button* m_editElementsImBut[26];
				Fl_Output* m_editElementsIndex[26];
				Fl_Button* m_editElementsDup[26];
				Fl_Button* m_editElementsDel[26];

		//loops tabs
		Fl_Group *m_loopsTab;
		Fl_Group* m_loopsTabButton;
		TardiButton* m_execCurrentLoopEraseButton;
		Fl_Dial* m_execCurrentLoopPos;
		std::vector<GrilleGroup*> m_execLoops;


		//stored values for lock-free modifications of widgets
		bool m_execCurrentLoopPosChanged;
		float m_execCurrentLoopPosValue;
		bool m_selectedExecGrilleChanged;
		int m_selectedExecGrilleValue;
		bool m_resetExecGrillesAsked;
		bool m_resetInputsTabAsked;
		bool m_resetAsked;
		bool m_reinitGLAsked;
		bool m_refreshAllPreviews;
		bool m_setTitleAsked;
		std::string m_setTitle;
		bool m_execMasterSliderChanged;
		float m_execMasterSliderValue;
		bool m_openFile;
		std::string m_openFileName;

		static const int MENU_HEIGHT=30;
		static const int EXEC_GRILLE_LIST_Y=20;
		static const int EXEC_GRILLE_LIST_W=70;
		static const int EXEC_GRILLE_GROUP_H=100;
		static const int EXEC_PARAMS_W=100;
		static const int EXEC_PARAMS_H=10;
		static const int EXEC_PARAMS_X=140;
		static const int EXEC_PARAMS_Y=40;
		static const int WIDGETS_SEP=5;
};

#endif /* _EXECWINDOW_H */
