/***************************************************************************
 *            GrilleGroup.cpp
 *
 *  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "GrilleGroup.hpp"
#include "../base/Vjpirate.hpp"
#include <iostream>
#include <math.h>

using namespace std;


GrilleGroup::GrilleGroup(int x, int y, int w,int h,const char* titre, GRILLE_GROUP_TYPE type, int grilleID, ExecWindow* execWin):Fl_Group(x,y,w,h,""), m_type(type), m_grilleID(grilleID), m_execWin(execWin), m_loopPosChanged(false), m_loopStateChanged(false), m_selected(false)
{
	//create the preview
	if(type==LOOP_GROUP) {
		m_preview = new RenderWindow(x+w*0.05, y+w*0.05, w*0.9, (w*0.9/4)*3, "", RENDER_LOOP);
	}
	else {
		m_preview = new RenderWindow(x+25, y+20, w-50, (w-50)/4*3, "", RENDER_GRILLE);
	}

	m_preview->setGrille(grilleID);

	box(FL_FLAT_BOX);
	color(FL_BACKGROUND2_COLOR);

	m_labelInput = new Fl_Input(x+10, m_preview->y()+m_preview->h(), w-20, 15,"");
	m_labelInput->value(titre);
	m_labelInput->textcolor(FL_WHITE);
	m_labelInput->cursor_color(FL_SELECTION_COLOR);
	m_labelInput->when(FL_WHEN_ENTER_KEY_ALWAYS);
	this->add(m_labelInput);
	m_labelInput->hide();

	m_label = new Fl_Button(x+10, m_preview->y()+m_preview->h(), w-20, 15,titre);
	m_label->box(FL_NO_BOX);
	m_label->clear_visible_focus();
	this->add(m_label);

	if(m_type == LOOP_GROUP) {
		//set the title (number of the loop)
		ostringstream oss;
		oss<<grilleID+1;
		m_label->copy_label(oss.str().c_str());
		//create the loop buttons
		m_playButton = new TardiButton(x+w-30,m_preview->y()+m_preview->h(),15,15,"@>");
		m_playButton->labelsize(10);
		m_playButton->box(FL_NO_BOX);
		m_playButton->labelcolor(FL_WHITE);
		m_playButton->clear_visible_focus();
		m_playButton->callback(statLoopButtons,this);
		m_eraseButton = new TardiButton(x+w-15,m_preview->y()+m_preview->h(),15,15,"X");
		m_eraseButton->labelsize(10);
		m_eraseButton->box(FL_NO_BOX);
		m_eraseButton->labelcolor(FL_RED);
		m_eraseButton->clear_visible_focus();
		m_eraseButton->callback(statLoopButtons,this);
		m_transparency = new TardiValueOutput(x+45,y+h-30,25,15,"Transp : ");
		m_transparency->align(FL_ALIGN_LEFT);
		m_transparency->box(FL_FLAT_BOX);
		m_transparency->color(FL_BACKGROUND2_COLOR);
		m_transparency->labelsize(8);
		m_transparency->textsize(8);
		m_transparency->bounds(0,100);
		m_transparency->step(1);
		m_transparency->clear_visible_focus();
		m_transparency->callback(statLoopButtons,this);
		m_speed = new TardiValueOutput(x+45,y+h-15,25,15,"Speed : ");
		m_speed->align(FL_ALIGN_LEFT);
		m_speed->box(FL_FLAT_BOX);
		m_speed->color(FL_BACKGROUND2_COLOR);
		m_speed->labelsize(8);
		m_speed->textsize(8);
		m_speed->bounds(50,200);
		m_speed->value(100);
		m_speed->step(1);
		m_speed->clear_visible_focus();
		m_speed->callback(statLoopButtons,this);
		m_pos = new Fl_Dial(x+w-30, y+h-25, 20, 20, "");
		m_pos->type(FL_FILL_DIAL);
		m_pos->angle1(-90);
		m_pos->angle2(270);
		m_pos->color(FL_WHITE);
		m_pos->color2(FL_SELECTION_COLOR);
		m_pos->clear_visible_focus();
		this->add(m_eraseButton);
		this->add(m_playButton);
		this->add(m_transparency);
		this->add(m_speed);
		this->add(m_pos);
		m_eraseButton->hide();
		m_playButton->hide();
		m_transparency->hide();
		m_speed->hide();
		m_pos->hide();
	}
	else if(type==GRILLE_GROUP) {

		m_labelInput->callback(statLabel,this);
		//m_labelInput->when(FL_WHEN_ENTER_KEY);
		m_label->callback(statLabel,this);

		m_grilleEraseButton = new Fl_Button(x+w-40, y, 15, 15, "x");
		m_grilleEraseButton->box(FL_NO_BOX);
		m_grilleEraseButton->labelcolor(FL_WHITE);
		m_grilleEraseButton->clear_visible_focus();
		m_grilleEraseButton->hide();
		m_grilleEraseButton->callback(statGrilleButtons,this);
		
		m_grilleDuplicateButton = new Fl_Button(x+w-20, y, 15, 15, "+");
		m_grilleDuplicateButton->box(FL_NO_BOX);
		m_grilleDuplicateButton->labelcolor(FL_WHITE);
		m_grilleDuplicateButton->clear_visible_focus();
		m_grilleDuplicateButton->hide();
		m_grilleDuplicateButton->callback(statGrilleButtons,this);

		m_grilleMoveLeftButton = new Fl_Button(x, y, 15, 15, "@-4<-");
		m_grilleMoveLeftButton->box(FL_NO_BOX);
		m_grilleMoveLeftButton->labelcolor(FL_WHITE);
		m_grilleMoveLeftButton->clear_visible_focus();
		m_grilleMoveLeftButton->hide();
		m_grilleMoveLeftButton->callback(statGrilleButtons,this);
		m_grilleMoveRightButton = new Fl_Button(x+15, y, 15, 15, "@-4->");
		m_grilleMoveRightButton->box(FL_NO_BOX);
		m_grilleMoveRightButton->labelcolor(FL_WHITE);
		m_grilleMoveRightButton->clear_visible_focus();
		m_grilleMoveRightButton->hide();
		m_grilleMoveRightButton->callback(statGrilleButtons,this);

		this->add(m_grilleEraseButton);
		this->add(m_grilleDuplicateButton);
		this->add(m_grilleMoveLeftButton);
		this->add(m_grilleMoveRightButton);
	}

	end();
}

GrilleGroup::~GrilleGroup()
{
	remove(m_preview);
	m_preview->clear();
	delete m_preview;

	Fl_Group::clear();
}

int GrilleGroup::size(int width, int height)
{
	Fl_Group::size(width,height);
/*

	if(m_type==GRILLE_GROUP) {
		if(m_selected) {
			m_preview->position(x(), y());
			m_preview->size(((h()-10)/3)*4, h()-10);
		}
		else {
			m_preview->position(x()+w()*0.05, y()+w()*0.05);
			m_preview->size((h()/6)*4, h()/2);
		}

		m_labelInput->position(x()+w(), y()+h()-15);
		m_grilleEraseButton->position(x(),y());
		m_grilleDuplicateButton->position(x()+w()-15, y());
		m_grilleMoveRightButton->position(x()+w()-15, y()+h()-15);
		m_grilleMoveLeftButton->position(x()+w()-30, y()+h()-15);
	}
*/
}

int GrilleGroup::handle(int event)
{
	switch(event) {
		case FL_PUSH: {
			if (Fl_Group::handle(event)) {
				return 1;
			}
			do_callback();
		}break;
		case FL_ENTER:{
			edited(true);	
			return 1;
		}break;
		case FL_LEAVE:{
			edited(false);	
			return 1;
		}break;
		default:{
			if (Fl_Group::handle(event)) {
				return 1;
			}
		
		}
	}

	return 0;
}

void GrilleGroup::edited(const bool& ed)
{
	if(m_type == GRILLE_GROUP) {
		if(ed) {
			m_grilleEraseButton->show();
			m_grilleDuplicateButton->show();
			m_grilleMoveLeftButton->show();
			m_grilleMoveRightButton->show();
		}
		else {
			m_grilleEraseButton->hide();
			m_grilleDuplicateButton->hide();
			m_grilleMoveLeftButton->hide();
			m_grilleMoveRightButton->hide();
		}
	}
	redraw();
}

void GrilleGroup::selected(const bool& sel)
{
	m_selected=sel;
	if(sel) {
		color(FL_SELECTION_COLOR);
	}
	else {
		color(FL_BACKGROUND2_COLOR);
	}
	redraw();
}


void GrilleGroup::setLoopState(const LOOP_STATE& state)
{
	switch(state) {
		case LOOP_EMPTY:{
			m_eraseButton->hide();
			m_playButton->hide();
			m_transparency->hide();
			m_speed->hide();
			m_pos->hide();
			color(FL_INACTIVE_COLOR);
		}break;
		case LOOP_RECORDING:{
			color(FL_RED);
		}break;
		case LOOP_OVERDUBBING:{
			color(FL_RED);
		}break;
		case LOOP_PLAYING:{ 
			m_playButton->label("@>");
			m_eraseButton->show();
			m_playButton->show();
			m_transparency->color(FL_SELECTION_COLOR);
			m_transparency->show();
			m_speed->color(FL_SELECTION_COLOR);
			m_speed->show();
			m_pos->show();
			color(FL_SELECTION_COLOR);
		}break;
		case LOOP_STOPPED:{
			m_playButton->label("@||");
			m_eraseButton->show();
			m_playButton->show();
			m_transparency->color(FL_INACTIVE_COLOR);
			m_transparency->show();
			m_speed->color(FL_INACTIVE_COLOR);
			m_speed->show();
			m_pos->show();
			color(FL_INACTIVE_COLOR);
		}break;
	}
}

void GrilleGroup::cbLabel(Fl_Widget* w)
{
	Vjpirate* vj = Vjpirate::getInstance();
	if(m_label->contains(w)) {
		m_labelInput->value(std::string(m_label->label()).c_str());
		m_label->hide();
		m_labelInput->show();
	}
	else if(m_labelInput->contains(w)) {
		m_label->copy_label(std::string(m_labelInput->value()).c_str());
		m_labelInput->hide();
		m_label->show();
		fl_cursor(FL_CURSOR_DEFAULT);
		vj->renameGrille(m_grilleID, std::string(m_labelInput->value()));	
	}
	
}

void GrilleGroup::cbLoopButtons(Fl_Widget* w)
{
	Vjpirate* vj = Vjpirate::getInstance();

	
	if(m_playButton->contains(w)) {
		if(Fl::event_button()==FL_LEFT_MOUSE) {
			vj->triggerLoop(LOOP_ACTION_TRIGGER, m_grilleID);	
		}
		else if(Fl::event_button()==FL_RIGHT_MOUSE) {
			std::vector<int> args(4,0);
			args[0]=-1;
			args[1]=LOOP_INPUT;
			args[2]=m_grilleID;
			args[3]=LOOP_TRIGGER;
			m_execWin->openLearnMenu(args);		
		}
	}
	else if(m_eraseButton->contains(w)) {
		if(Fl::event_button()==FL_LEFT_MOUSE) {
			vj->eraseLoop(m_grilleID);	
		}
		else if(Fl::event_button()==FL_RIGHT_MOUSE) {
			std::vector<int> args(4,0);
			args[0]=-1;
			args[1]=LOOP_INPUT;
			args[2]=m_grilleID;
			args[3]=LOOP_ERASE;
			m_execWin->openLearnMenu(args);		
		}
	}
	else if(m_transparency->contains(w)) {
		if(Fl::event_button()==FL_LEFT_MOUSE) {
			vj->setLoopTransparency(m_grilleID,m_transparency->value()/100.0);	
		}
		else if(Fl::event_button()==FL_RIGHT_MOUSE) {
			std::vector<int> args(4,0);
			args[0]=-1;
			args[1]=LOOP_INPUT;
			args[2]=m_grilleID;
			args[3]=LOOP_TRANSPARENCY;
			m_execWin->openLearnMenu(args);		
		}
	}
	else if(m_speed->contains(w)) {
		if(Fl::event_button()==FL_LEFT_MOUSE) {
			vj->setLoopSpeed(m_grilleID,m_speed->value()/100.0,false);	
		}
		else if(Fl::event_button()==FL_RIGHT_MOUSE) {
			std::vector<int> args(4,0);
			args[0]=-1;
			args[1]=LOOP_INPUT;
			args[2]=m_grilleID;
			args[3]=LOOP_SPEED;
			m_execWin->openLearnMenu(args);		
		}
	}
}

void GrilleGroup::cbGrilleButtons(Fl_Widget* w)
{
	Vjpirate* vj = Vjpirate::getInstance();
	
	if(m_grilleEraseButton->contains(w)) {
		vj->removeGrille(m_grilleID);
	}
	else if(m_grilleDuplicateButton->contains(w)) {
		vj->duplicateGrille(m_grilleID);
	}
	else if(m_grilleMoveRightButton->contains(w)) {
		vj->swapGrilles(m_grilleID, m_grilleID+1);
	}
	else if(m_grilleMoveLeftButton->contains(w)) {
		vj->swapGrilles(m_grilleID, m_grilleID-1);
	}
}

void GrilleGroup::redraw() {
    Fl_Group::redraw();
    m_preview->redraw();
}

void GrilleGroup::updateValues()
{
	m_speed->updateValue();
	m_transparency->updateValue();

	if(m_loopPosChanged) {
		m_pos->value(m_loopPosValue);
		m_loopPosChanged=false;
		m_pos->redraw();
	}

	if(m_loopStateChanged) {
		setLoopState(m_loopStateValue);
		m_loopStateChanged=false;
		this->redraw();
	}
}

