/***************************************************************************
 *            GrilleGroup.h
 *
 *  Part of VjPirate
 *  2010  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <FL/Fl.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Counter.H>
#include <FL/Fl_Dial.H>
#include <FL/Fl_Value_Output.H>
#include <iostream>
#include <string>
#include <vector>

#include "TardiButton.hpp"
#include "TardiValueOutput.hpp"
 
#ifndef _GRILLEGROUP_H
#define _GRILLEGROUP_H

#include "RenderWindow.hpp"

enum GRILLE_GROUP_TYPE { LOOP_GROUP, GRILLE_GROUP};

class ExecWindow;

class GrilleGroup : public Fl_Group
{
  public:
		GrilleGroup(int ,int ,int,int,const char*, GRILLE_GROUP_TYPE, int, ExecWindow*);
		~GrilleGroup();

		inline void hide(){m_preview->hide();}
		inline void show(){m_preview->show();}
		void redraw();
		inline void redrawPreview(){m_preview->redraw();this->redraw();}

		static void statLabel(Fl_Widget* w,void* f){GrilleGroup *tmpf = static_cast<GrilleGroup *>(f);tmpf->cbLabel(w);}	
		void cbLabel(Fl_Widget*);
		static void statLoopButtons(Fl_Widget* w,void* f){GrilleGroup *tmpf = static_cast<GrilleGroup *>(f);tmpf->cbLoopButtons(w);}	
		void cbLoopButtons(Fl_Widget*);
		static void statGrilleButtons(Fl_Widget* w,void* f){GrilleGroup *tmpf = static_cast<GrilleGroup *>(f);tmpf->cbGrilleButtons(w);}	
		void cbGrilleButtons(Fl_Widget*);

		int handle(int);	
		int size(int,int);
		
		inline void setLabel(const std::string& lab){m_label->label(lab.c_str()); m_labelInput->value(lab.c_str());};

		void selected(const bool&);
		void edited(const bool&);
		inline const bool& getSelected(){return m_selected;}
		void setLoopState(const LOOP_STATE&);
		inline void setLoopStateValue(const LOOP_STATE& state){m_loopStateValue=state;m_loopStateChanged=true;}
		inline void setLoopSpeed(const float& val){m_speed->setValue(val);}
		inline void setLoopTransparency(const float& val){m_transparency->setValue(val);}
		inline void setLoopPos(const float& val){m_loopPosValue=val;m_loopPosChanged=true;}

		void updateValues();

  private:

		RenderWindow *m_preview;
		ExecWindow* m_execWin;
		GRILLE_GROUP_TYPE m_type;
		int m_grilleID;

		Fl_Input *m_labelInput;
		Fl_Button *m_label;

		TardiButton* m_playButton;
		TardiButton* m_eraseButton;
		TardiValueOutput* m_transparency;
		TardiValueOutput* m_speed;
		Fl_Dial* m_pos;

		bool m_loopPosChanged;
		float m_loopPosValue;

		bool m_loopStateChanged;
		LOOP_STATE m_loopStateValue;
		bool m_selected;

		Fl_Button * m_grilleEraseButton;
		Fl_Button * m_grilleDuplicateButton;
		Fl_Button * m_grilleMoveLeftButton;
		Fl_Button * m_grilleMoveRightButton;
};

#endif

