/***************************************************************************
 *            GrilleScroll.cpp
 *
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "GrilleScroll.hpp"
#include "../base/Vjpirate.hpp"
#include "ExecWindow.hpp"
#include <iostream>
#include <math.h>

using namespace std;


GrilleScroll::GrilleScroll(int x, int y, int w,int h,const char* titre):Fl_Group(x,y,w,h,titre), m_sliderPos(0), m_sliderSize(w), m_childrenTotalSize(0), m_mouseGrabbed(false), m_mouseX(0), m_selectedGrille(0)
{
	m_childrenWidth=(h/6)*4;


	end();
}

GrilleScroll::~GrilleScroll()
{
}

void GrilleScroll::selectGrille(const int& gri)
{
	m_selectedGrille=gri;
	//select the grille, deselect the other and move the scrollbar
	for(unsigned int i=0;i<children();++i) {
		((GrilleGroup*)child(i))->selected(gri==i);
	}
	m_sliderPos=float((w()-2*SCROLL_SEP)-m_sliderSize)*(float(gri)/float(children()));
	m_sliderPos=(m_sliderPos<0)?0:m_sliderPos;
	redraw();
	for(unsigned int i=0;i<children();++i) {
        child(i)->redraw();
    }
}

void GrilleScroll::add(Fl_Widget* widget)
{
	Fl_Group::add(widget);
	m_childrenTotalSize+=widget->w()+SCROLL_SEP;			
	if(m_childrenTotalSize>w()-2*SCROLL_SEP) {
		m_sliderSize=float(w()-2*SCROLL_SEP)*(float(w()-2*SCROLL_SEP)/float(m_childrenTotalSize));
	}
	else {
		m_sliderSize=w()-2*SCROLL_SEP;
	}
}

void GrilleScroll::remove(Fl_Widget* widget)
{
	Fl_Group::remove(widget);
	m_childrenTotalSize-=(widget->w()+SCROLL_SEP);			
	if(m_childrenTotalSize>w()-2*SCROLL_SEP) {
		m_sliderSize=float(w()-2*SCROLL_SEP)*(float(w()-2*SCROLL_SEP)/float(m_childrenTotalSize));
	}
	else {
		m_sliderSize=w()-2*SCROLL_SEP;
	}
}

void GrilleScroll::clear()
{
	Fl_Group::clear();
	
	m_childrenTotalSize=0;			
	m_sliderSize=w()-2*SCROLL_SEP;
	m_sliderPos=0;
}

int GrilleScroll::handle(int event)
{

	//if left click
	if(event==FL_PUSH && Fl::event_button()==FL_RIGHT_MOUSE) {
		do_callback();
	}
	else {
		//test the widgets
		if(Fl_Group::handle(event)) {
			return 1;
		}
	}

	if(m_sliderSize<w()-SCROLL_SEP*2) {

		int mouseX = Fl::event_x();
		int mouseY = Fl::event_y();
		int sliderXMin = x()+m_sliderPos;
		int sliderXMax = sliderXMin+m_sliderSize;
		int sliderYMin = y()+h()-SLIDER_HEIGHT;
		int sliderYMax = sliderYMin+SLIDER_HEIGHT;

		//test if pushed in the scrollbar area
		switch(event) {
			case FL_PUSH: {
				if(mouseX>sliderXMin && mouseX<sliderXMax && mouseY>sliderYMin && mouseY<sliderYMax) {
					m_mouseGrabbed=true;
					m_mouseX=mouseX;
				}
				return 1;
			}break;
			case FL_DRAG:{
				int t = Fl::event_inside(this);
				if(m_mouseGrabbed) {
					int newSliderPos = m_sliderPos+(mouseX-m_mouseX);
					if(newSliderPos+m_sliderSize>(w()-2*SCROLL_SEP)) {
						m_sliderPos=(w()-2*SCROLL_SEP)-m_sliderSize;
					}
					else if(newSliderPos<0) {
						m_sliderPos=0;
					}
					else {
						m_sliderPos=newSliderPos;
					}
					m_mouseX=mouseX;
					damage(FL_DAMAGE_USER1);
					m_selectedGrille = float(m_sliderPos)/float((w()-2*SCROLL_SEP)-m_sliderSize) * float(children()-1);
				}
				return 1;
			}break;
			case FL_RELEASE:{
				m_mouseGrabbed=false;
				return 1;
			}break;
		}	
	}
	return 0;
}

void GrilleScroll::draw()
{
	//draw the box
	fl_color(FL_BACKGROUND2_COLOR);
	fl_rectf(x(),y(),w(),h());	

	//draw the label
	draw_label();

	//draw the children
	//int posX= w()/2 + x()+(-m_selectedGrille*m_childrenWidth) - m_childrenWidth;
	//int posY=y();
	for(unsigned int i=0;i<children();++i) {
		int posX = x() + (i-m_selectedGrille)*child(i)->w() + w()/2 - child(i)->w()/2;
		child(i)->position(posX, y());
		/*
		if(i==m_selectedGrille) {
			child(i)->position(posX, posY);
			((GrilleGroup*)child(i))->size(m_childrenWidth*2, h());
			posX+=m_childrenWidth*2;
		}
		else {
			child(i)->position(posX, posY);
			((GrilleGroup*)child(i))->size(m_childrenWidth, h()/2);
			posX+=m_childrenWidth;
		}
		*/
		child(i)->draw();
	}

	//draw the scrollbar
	fl_color(FL_INACTIVE_COLOR);
	fl_rectf(x()+SCROLL_SEP,y()+h()-SLIDER_HEIGHT,w()-2*SCROLL_SEP,SLIDER_HEIGHT);	
	fl_color(fl_darker(FL_INACTIVE_COLOR));
	fl_rectf(x()+SCROLL_SEP+m_sliderPos,y()+h()-SLIDER_HEIGHT,m_sliderSize,SLIDER_HEIGHT);	
}

