/***************************************************************************
 *            InputGroup.cpp
 *
 *  2007  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include "InputGroup.hpp"

#include "../base/Vjpirate.hpp"

using namespace std;


InputGroup::InputGroup(int x, int y, int w, int h):Fl_Group(x,y,w,h,"")
{
	color(FL_INACTIVE_COLOR);
	box(FL_FLAT_BOX);
	m_group = new Fl_Choice(x,y,CHOICE_WIDTH,h,"");
	m_group->labelsize(8);
	m_group->box(FL_FLAT_BOX);
	m_group->down_box(FL_FLAT_BOX);
	m_group->selection_color(FL_SELECTION_COLOR);
	m_group->clear_visible_focus();
	m_group->add("Grilles");
	m_group->add("Loops");
	m_group->add("Application");
	m_group->callback(statInput,this);
	m_grille = new Fl_Choice(m_group->x()+m_group->w()+WIDGETS_SEP,y,CHOICE_WIDTH,h,"");
	m_grille->add("Selected Grille");
	m_grille->value(0);
	m_grille->labelsize(8);
	m_grille->box(FL_FLAT_BOX);
	m_grille->selection_color(FL_SELECTION_COLOR);
	m_grille->clear_visible_focus();
	m_grille->callback(statInput,this);
	m_parameter = new Fl_Choice(m_grille->x()+m_grille->w()+WIDGETS_SEP,y,CHOICE_WIDTH,h,"");
	m_parameter->add("Columns");
	m_parameter->value(0);
	m_parameter->labelsize(8);
	m_parameter->box(FL_FLAT_BOX);
	m_parameter->selection_color(FL_SELECTION_COLOR);
	m_parameter->clear_visible_focus();
	m_parameter->callback(statInput,this);

	setGroup(0);

	m_type = new Fl_Choice(m_parameter->x()+m_parameter->w()+WIDGETS_SEP,y,CHOICE_WIDTH,h,"");
	m_type->labelsize(8);
	m_type->box(FL_FLAT_BOX);
	m_type->selection_color(FL_SELECTION_COLOR);
	m_type->clear_visible_focus();
	for(unsigned int i=0;i<INPUT_TYPES_NB;++i) {
		switch(INPUT_TYPE(i)) {
			case INPUT_MIDI_CC: {
				m_type->add("MIDI CC");
			}break;
			case INPUT_MIDI_NOTE: {
				m_type->add("MIDI Note");
			}break;
			case INPUT_MIDI_NOTE_RANGE: {
				m_type->add("MIDI Note Range");
			}break;
			case INPUT_OSC: {
				m_type->add("OpenSoundControl");
			}break;
			case INPUT_AUDIO: {
				m_type->add("Audio");
			}break;
			case INPUT_KEYBOARD: {
				m_type->add("Keyboard");
			}break;
            default:break;
		}
	}
	m_type->callback(statInput,this);
	m_midiChannel = new Fl_Value_Input(m_type->x()+m_type->w()+WIDGETS_SEP*2, y, INPUT_WIDTH,h, "");
	m_midiChannel->box(FL_FLAT_BOX);
	m_midiChannel->labelsize(8);
	m_midiChannel->selection_color(FL_SELECTION_COLOR);
	m_midiChannel->clear_visible_focus();
	m_midiChannel->bounds(0,15);
	m_midiChannel->value(0);
	m_midiChannel->step(1);
	m_midiChannel->callback(statInput,this);
	m_midiCC = new Fl_Value_Input(m_midiChannel->x()+m_midiChannel->w()+WIDGETS_SEP+20, y, INPUT_WIDTH2, h, "");
	m_midiCC->box(FL_FLAT_BOX);
	m_midiCC->labelsize(8);
	m_midiCC->selection_color(FL_SELECTION_COLOR);
	m_midiCC->clear_visible_focus();
	m_midiCC->bounds(0,127);
	m_midiCC->value(0);
	m_midiCC->step(1);
	m_midiCC->callback(statInput,this);
	m_midiNote = new Fl_Value_Input(m_midiChannel->x()+m_midiChannel->w()+WIDGETS_SEP+20,y,INPUT_WIDTH2,h,"");
	m_midiNote->labelsize(8);
	m_midiNote->box(FL_FLAT_BOX);
	m_midiNote->selection_color(FL_SELECTION_COLOR);
	m_midiNote->clear_visible_focus();
	m_midiNote->hide();
	m_midiNote->bounds(0,127);
	m_midiNote->value(0);
	m_midiNote->step(1);
	m_midiNote->callback(statInput,this);
	m_midiNoteMax = new Fl_Value_Input(m_midiNote->x()+m_midiNote->w()+WIDGETS_SEP+20,y,INPUT_WIDTH2,h,"");
	m_midiNoteMax->box(FL_FLAT_BOX);
	m_midiNoteMax->labelsize(8);
	m_midiNoteMax->selection_color(FL_SELECTION_COLOR);
	m_midiNoteMax->clear_visible_focus();
	m_midiNoteMax->hide();
	m_midiNoteMax->bounds(0,127);
	m_midiNoteMax->value(0);
	m_midiNoteMax->step(1);
	m_midiNoteMax->callback(statInput,this);
	m_oscString = new Fl_Input(m_type->x()+m_type->w()+WIDGETS_SEP*2, y, TEXT_INPUT_WIDTH, h, "");
	m_oscString->labelsize(8);
	m_oscString->box(FL_FLAT_BOX);
	m_oscString->selection_color(FL_SELECTION_COLOR);
	m_oscString->value("/message");
	m_oscString->when(FL_WHEN_ENTER_KEY_ALWAYS);
	m_oscString->hide();
	m_oscString->callback(statInput,this);
	m_key = new Fl_Input(m_type->x()+m_type->w()+WIDGETS_SEP*2, y, 20, h, "");
	m_key->labelsize(8);
	m_key->box(FL_FLAT_BOX);
	m_key->when(FL_WHEN_CHANGED);
	m_key->selection_color(FL_SELECTION_COLOR);
	m_key->value("a");
	m_key->hide();
	m_key->callback(statInput,this);
	m_audioFeature = new Fl_Choice(m_type->x()+m_type->w()+WIDGETS_SEP,y,100,h,"");
	m_audioFeature->labelsize(8);
	m_audioFeature->box(FL_FLAT_BOX);
	m_audioFeature->selection_color(FL_SELECTION_COLOR);
	m_audioFeature->clear_visible_focus();
	m_audioFeature->add("Loudness");
	m_audioFeature->add("Bass");
	m_audioFeature->add("Medium");
	m_audioFeature->add("High");
	m_audioFeature->add("Noisiness");
	m_audioFeature->add("Brightness");
	m_audioFeature->add("Onset");
	m_audioFeature->value(0);
	m_audioFeature->callback(statInput,this);
	m_mode = new Fl_Choice(m_midiNoteMax->x()+m_midiNoteMax->w()+WIDGETS_SEP,y,100,h,"");
	m_mode->labelsize(8);
	m_mode->box(FL_FLAT_BOX);
	m_mode->selection_color(FL_SELECTION_COLOR);
	for(unsigned int i=0;i<INPUT_MODES_NB;++i) {
		switch(INPUT_MODE(i)) {
			case MODE_NORMAL: {
				m_mode->add("Normal");
			}break;
			case MODE_TOGGLE: {
				m_mode->add("Toggle");
			}break;
			case MODE_STEP_NORMAL: {
				m_mode->add("Step Normal");
			}break;
			case MODE_STEP_CIRCULAR: {
				m_mode->add("Step Circular");
			}break;
			case MODE_STEP_PING: {
				m_mode->add("Step Ping Pong");
			}break;
			case MODE_STEP_RANDOM: {
				m_mode->add("Step Random");
			}break;
            default:break;
		}
	}
	m_mode->callback(statInput,this);
	m_step = new Fl_Value_Input(m_mode->x()+m_mode->w()+WIDGETS_SEP,y,INPUT_WIDTH2,h,"");
	m_step->box(FL_FLAT_BOX);
	m_step->labelsize(8);
	m_step->selection_color(FL_SELECTION_COLOR);
	m_step->clear_visible_focus();
	m_step->hide();
	m_step->bounds(0.01,0.5);
	m_step->value(0.1);
	m_step->step(0.01);
	m_step->callback(statInput,this);

	setType(0);

	m_learn = new Fl_Toggle_Button(x + w - BUTTON_WIDTH*2 - WIDGETS_SEP,y,BUTTON_WIDTH,h,"L");
	m_learn->box(FL_FLAT_BOX);
	m_learn->selection_color(FL_SELECTION_COLOR);
	m_learn->clear_visible_focus();
	m_learn->callback(statInput,this);
	m_add = new Fl_Button(x+w-20,y,15,h/2,"@-9+");
	m_add->box(FL_FLAT_BOX);
	m_add->selection_color(FL_SELECTION_COLOR);
	m_add->clear_visible_focus();
	m_add->callback(statInput,this);
	m_remove = new Fl_Button(x+w-20,y+h/2,15,h/2,"-");
	m_remove->box(FL_FLAT_BOX);
	m_remove->selection_color(FL_SELECTION_COLOR);
	m_remove->clear_visible_focus();
	m_remove->callback(statInput,this);
	end();
}

InputGroup::~InputGroup()
{
	this->clear();
}

void InputGroup::setInput(const Input& inp)
{
	setGroup(inp.group);
	setGrille(inp.grille);
	m_parameter->value(inp.parameter);
	setType(inp.type);
	switch(inp.type) {
		case INPUT_MIDI_CC: {
			m_midiChannel->value(inp.midiChannel);
			m_midiCC->value(inp.midiCC);
		}break;
		case INPUT_MIDI_NOTE: {
			m_midiNote->value(inp.midiNote);
			m_midiChannel->value(inp.midiChannel);
		}break;
		case INPUT_MIDI_NOTE_RANGE: {
			m_midiNote->value(inp.midiNote);
			m_midiNoteMax->value(inp.midiNoteMax);
			m_midiChannel->value(inp.midiChannel);
		}break;
		case INPUT_OSC: {
			m_oscString->value(inp.oscMessage.c_str());
		}
		break;
		case INPUT_AUDIO: {
			m_audioFeature->value(inp.audioFeature);
			setAudioFeature(inp.audioFeature);
		}break;
		case INPUT_KEYBOARD: {
			std::string strVal="";
			strVal+=inp.key;
			m_key->value(strVal.c_str());
		}break;
	}

	m_mode->value(inp.mode);
	m_step->hide();
	if(inp.mode>=MODE_STEP_NORMAL){
		m_step->show();
		m_step->value(inp.step);
	}

	//remove learning
	m_learn->value(0);
}



void InputGroup::setGroup(const int& group)
{
	Vjpirate *vjp = Vjpirate::getInstance();
	m_group->value(group);
	m_grille->clear();		
	m_parameter->clear();
	switch(group) {
		case GRILLE_INPUT :{	
			m_grille->add("Selected Grille");
			for(unsigned int i=0;i<vjp->getNbGrilles();++i) {
				m_grille->add(vjp->getGrille(i).getName().c_str());
			}
			setGrille(0);
		}break;
		case LOOP_INPUT : {
			m_grille->add("Loop 1");
			m_grille->add("Loop 2");
			m_grille->add("Loop 3");
			m_grille->add("Loop 4");
			m_grille->add("Loop 5");
			m_grille->add("Loop 6");
			m_grille->add("Loop 7");
			m_grille->add("Loop 8");
			setGrille(0);
		}break;
		case APPLICATION_INPUT: {
			m_parameter->add("Master");
			m_parameter->add("Selection");
			m_parameter->add("Selection Activation");
			m_parameter->add("Loop Record");
			m_parameter->add("Loop Erase");
			m_parameter->value(0);
		}break;
	}
}

void InputGroup::setGrille(const int& gri)
{
	m_grille->value(gri);
	switch(m_group->value()) {
		case GRILLE_INPUT :{
			m_parameter->clear();
			Parameter param;
			//add parameters names
			for(unsigned int p=0; p<PARAMS_NB; ++p) {
				std::string name;
				param.getNameFromId(p,name);
				m_parameter->add(name.c_str());
			}

			//add permut names
			for(unsigned int pp=PARAM_SIZE;pp<PARAMS_NB;++pp) {
				std::string name;
				param.getNameFromId(pp,name);
				name +=" permutation";
				m_parameter->add(name.c_str());
			}

			m_parameter->add("Activation");
			m_parameter->add("Reinitialization");
			m_parameter->add("Blend");
			m_parameter->add("Audio Activation");
		}break;
		case LOOP_INPUT: {
			m_parameter->clear();
			m_parameter->add("Trigger");
			m_parameter->add("Erase");
			m_parameter->add("Speed");
			m_parameter->add("Transparency");
			m_parameter->add("Erase Toggle");
		}break;
	}

	m_parameter->value(0);

}

void InputGroup::setType(const int& type)
{
	m_oscString->hide();
	m_midiNote->hide();
	m_midiNoteMax->hide();
	m_midiCC->hide();
	m_midiChannel->hide();
	m_audioFeature->hide();
	m_key->hide();

	m_type->value(type);
	m_step->hide();

	switch(type) {
		case INPUT_MIDI_CC :{
			m_midiChannel->show();
			m_midiCC->show();
		}break;
		case INPUT_MIDI_NOTE :{
			m_midiChannel->show();
			m_midiNote->show();
		}break;
		case INPUT_MIDI_NOTE_RANGE :{
			m_midiChannel->show();
			m_midiNote->show();
			m_midiNoteMax->show();
		}break;
		case INPUT_OSC :{
			m_oscString->show();
		}break;
		case INPUT_AUDIO :{
			m_audioFeature->show();
			m_audioFeature->value(0);
		}break;
		case INPUT_KEYBOARD :{
			m_key->value("a");
			m_key->show();
		}break;
	}
}

void InputGroup::setAudioFeature(const int& feat)
{
	m_audioFeature->value(feat);

}

void InputGroup::setMode(const int& mode)
{
	m_mode->value(mode);
	m_step->hide();
	if(mode>=MODE_STEP_NORMAL) {
		m_step->value(0.1);
		m_step->show();
	}
}

void InputGroup::cbInput(Fl_Widget* w)
{
	Vjpirate *vjp = Vjpirate::getInstance();
	Input modifiedInput;

	if(m_group->contains(w)) {
		setGroup(m_group->value());
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_grille->contains(w)) {
		int parameter=m_parameter->value();
		setGrille(m_grille->value());
		if(parameter<m_parameter->size()) {
			m_parameter->value(parameter);
		}
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_parameter->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_type->contains(w)) {
		setType(m_type->value());
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_midiChannel->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_midiCC->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_midiNote->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_midiNoteMax->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_oscString->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_audioFeature->contains(w)) {
		setAudioFeature(m_audioFeature->value());
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_mode->contains(w)) {
		setMode(m_mode->value());
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_step->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_key->contains(w)) {
		std::string keyStr = m_key->value();
		if(keyStr.size()>0) {
			m_key->value(keyStr.substr(keyStr.size()-1,1).c_str());
		}
		else {
			m_key->value("a");
		}
		m_mode->take_focus();	
		getInputFromValues(modifiedInput);
		vjp->setInput(m_inputID,modifiedInput);
	}
	else if(m_learn->contains(w)) {
		std::vector<int> args(4,0);
		args[0]=m_inputID;
		args[1]=m_group->value();
		args[2]=m_grille->value();
		args[3]=m_parameter->value();
		vjp->learnInput(m_learn->value(), args);
	}
	else if(m_add->contains(w)) {
		getInputFromValues(modifiedInput);
		vjp->addInput(modifiedInput);
	}
	else if(m_remove->contains(w)) {
		vjp->removeInput(m_inputID);
	}
}

void InputGroup::getInputFromValues(Input& modifiedInput)
{
	modifiedInput.group=INPUT_GROUP(m_group->value());
	modifiedInput.grille=m_grille->value();
	modifiedInput.parameter=m_parameter->value();

	modifiedInput.type=INPUT_TYPE(m_type->value());
	modifiedInput.midiChannel=m_midiChannel->value();
	modifiedInput.midiCC=m_midiCC->value();
	modifiedInput.midiNote=m_midiNote->value();
	modifiedInput.midiChannel=m_midiChannel->value();
	modifiedInput.midiNoteMax=m_midiNoteMax->value();

	std::string keyStr = m_key->value();
	if(keyStr.length()>0) {
		modifiedInput.key=keyStr[0];
	}
	else {
		modifiedInput.key='a';
	}

	modifiedInput.oscMessage=m_oscString->value();
	modifiedInput.audioFeature=ANALYSIS_FEATURE(m_audioFeature->value());
	modifiedInput.mode=INPUT_MODE(m_mode->value());
	modifiedInput.step=m_step->value();

}

