/***************************************************************************
 *            InputGroup.h
 *
 *  2007  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Hold_Browser.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Value_Input.H>
#include <FL/filename.H>
#include <FL/fl_ask.H>
#include <iostream>
#include <cstring>
#include <vector>
#include <cstdlib>

#ifndef _INPUT_GROUP_H
#define _INPUT_GROUP_H

class Input;

class InputGroup : public Fl_Group
{
	public:
		InputGroup(int, int, int, int);
		~InputGroup();

		static void statInput(Fl_Widget* w,void* f){InputGroup *tmpf = static_cast<InputGroup *>(f);tmpf->cbInput(w);}	
		void cbInput(Fl_Widget*);

		inline void setInputID(int inputID){m_inputID=inputID;}
		void setInput(const Input&);
		void setGroup(const int&);
		void setGrille(const int&);
		void setMode(const int&);
		void setType(const int&);
		void setAudioFeature(const int&);

		
		void getInputFromValues(Input&);

	private:
		Fl_Choice* m_group;
		Fl_Choice* m_type;
		Fl_Choice* m_grille;
		Fl_Choice* m_parameter;
		Fl_Value_Input* m_midiChannel;
		Fl_Value_Input* m_midiCC;
		Fl_Value_Input* m_midiNote;
		Fl_Value_Input* m_midiNoteMax;
		Fl_Input* m_oscString;
		Fl_Input* m_key;
		Fl_Choice* m_audioFeature;
		Fl_Choice* m_mode;
		Fl_Value_Input* m_step;

		Fl_Toggle_Button* m_learn;
		Fl_Button* m_remove;
		Fl_Button* m_add;

		int m_inputID;

		static const int CHOICE_WIDTH=120;
		static const int INPUT_WIDTH=30;
		static const int INPUT_WIDTH2=40;
		static const int TEXT_INPUT_WIDTH=100;
		static const int BUTTON_WIDTH=20;
		static const int WIDGETS_SEP=5;

};

#endif
