/***************************************************************************
 *            RenderWindow.cpp
 *
 *  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "RenderWindow.hpp"
#include "../base/Vjpirate.hpp"
#include <iostream>
#include <FL/x.H>
#include <math.h>

using namespace std;


RenderWindow::RenderWindow(int w, int h, const char* titre, RENDER_WINDOW_TYPE type, int id):Fl_Gl_Window(w,h,titre), m_mouseButtonPressed(NO_BUTTON), m_resizable(true), m_fullscreen(false), m_type(type), m_grilleID(id), m_maxVerticesNb(1024), m_initialized(false)
{
	mode(FL_RGB|FL_DOUBLE|FL_ALPHA|FL_DEPTH|FL_MULTISAMPLE);
	box(FL_DOWN_BOX);
	end();
	m_width=this->w();
	m_height=this->h();
    resizable(this);
}

RenderWindow::RenderWindow(int x, int y, int w,int h,const char* titre, RENDER_WINDOW_TYPE type, int id):Fl_Gl_Window(x,y,w,h,titre), m_mouseButtonPressed(NO_BUTTON), m_resizable(false), m_fullscreen(false), m_type(type), m_grilleID(id), m_maxVerticesNb(1024), m_initialized(false)
{
	mode(FL_RGB|FL_DOUBLE|FL_ALPHA|FL_DEPTH|FL_MULTISAMPLE);
	box(FL_DOWN_BOX);
	cursor(FL_CURSOR_NONE);
	end();
	m_width=this->w();
	m_height=this->h();
    resizable(this);
}

RenderWindow::~RenderWindow()
{
	/*
	glDeleteBuffersARB(1, &m_vertexVboID);
	glDeleteBuffersARB(1, &m_colorsVboID);
	glDeleteBuffersARB(1, &m_texturesVboID);
	*/
}

void RenderWindow::setTitle(std::string t)
{
	label(t.c_str());	
}

void RenderWindow::draw()
{
	Vjpirate *vjp = Vjpirate::getInstance();

	m_width=this->w();
	m_height=this->h();
	
	//init
	if (!valid()) {

		glLoadIdentity();
		glViewport(0,0,w(),h());
		glOrtho(0,w(),h(),0,-1,1);
		//enable textures in opengl
		glEnable( GL_TEXTURE_2D);

		if(m_type==RENDER_MONITOR) {
			//load /unload images
			Vjpirate *vj=Vjpirate::getInstance();
			vj->testAllAnims();
		}
		
		glEnable (GL_LINE_SMOOTH);
		glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
   		glLineWidth (1.5);
		glEnable(GL_DEPTH_TEST);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
		glDepthFunc(GL_LEQUAL);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE);
		glEnable(GL_BLEND);
		glEnable(GL_MULTISAMPLE);
		glShadeModel(GL_FLAT);
		glClearColor(0,0,0,0);

		if(!m_initialized) {
			glGetIntegerv(GL_MAX_ELEMENTS_VERTICES, &m_maxVerticesNb);
			m_maxVerticesNb/=2;
			m_initialized=true;
		}

		/*
		if(m_type==RENDER_MONITOR) {
		
			//generate vbo's id's
			glGenBuffersARB(1, &m_vertexVboID);
			glGenBuffersARB(1, &m_colorsVboID);
			glGenBuffersARB(1, &m_texturesVboID);
			
			// bind VBO in order to use
			glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboId);
			
			// upload data to VBO
			glBufferDataARB(GL_ARRAY_BUFFER_ARB, dataSize, vertices, GL_STATIC_DRAW_ARB);
		}
		*/

		valid(1);
        }

	//clear
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	//enable vertex arrays
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);


	//get the grilles coordinates
	switch(m_type) {
		case RENDER_GRILLE : {
			if(m_grilleID>=0&&m_grilleID<vjp->getNbGrilles()) {
				Grille& grille = vjp->getGrille(m_grilleID);
				//if(grille.getActive()) {
					drawGrille(grille);
				//}
			}
		}break;
		case RENDER_LOOP : {
			if(m_grilleID>=0&&m_grilleID<vjp->getNbLoops()) {
				Grille& grille = vjp->getLoop(m_grilleID);
				if(grille.getLoopState()!=LOOP_EMPTY) {
					drawGrille(grille);
				}
			}
		}break;
		case RENDER_WINDOW:
		case RENDER_MONITOR :{

			/*
			glBindBufferARB(GL_ARRAY_BUFFER_ARB, m_vertexVboID);
			// upload data to VBO
			glBufferDataARB(GL_ARRAY_BUFFER_ARB, dataSize, vertices, GL_DYNAMIC_DRAW_ARB);
			*/
			//draw loops
			for(unsigned int i=0;i<vjp->getNbLoops();++i) {
				Grille& activeLoop = vjp->getLoop(i);
				if(activeLoop.getActive()) {
					drawGrille(activeLoop);
				}
			}

			//draw active Grilles
			for(unsigned int i=0;i<vjp->getNbGrilles();++i) {
				Grille& activeGrille = vjp->getGrille(i);
				if(activeGrille.getActive()) {
					drawGrille(activeGrille);
				}
			}
		}break;
	}


	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

/*
	GLenum errCode;
	const GLubyte *errString;

	if ((errCode = glGetError()) != GL_NO_ERROR) {
		errString = gluErrorString(errCode);
		printf ("OpenGL Error: %s\n", errString);
	}
*/
}

void RenderWindow::drawGrille(Grille& gri)
{

	//clear vertices maps
	//m_texturesVerticesMap.clear();

	m_nbRows=floor(gri.getParameter(PARAM_ROWS).getFirstElementValue());
	m_nbCols=floor(gri.getParameter(PARAM_COLS).getFirstElementValue());
	//cout<<"grille:"<<nbRows<<":"<<nbCols<<endl;
	/*
	m_cellWidth=(m_width/m_nbCols);
	m_cellHeight=(m_height/m_nbRows);
	*/
	m_cellWidth=(1/m_nbCols);
	m_cellHeight=(1/m_nbRows);
	if(gri.getHomothety()) {
		if(m_nbRows>m_nbCols) {
			m_cellWidth=1/m_nbRows;
		}
		else {
			m_cellHeight=1/m_nbCols;
		}
	}

	if(gri.getBlend()) {
		glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	}
	else {
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	}


	//compute the coords only once
	if(!gri.hasGlCoords()) {
		gri.clearGlCoords();
		//for each row and column, get the vertices / colors ... of the cells 
		m_cellCounter=0;
		for(m_rowCounter=0;m_rowCounter<m_nbRows;++m_rowCounter) {
			for(m_colCounter=0;m_colCounter<m_nbCols;++m_colCounter,++m_cellCounter) {
				//draw the cell
				drawCell(gri);
			}
		}
		gri.hasGlCoords(true);
	}

	

	//draw the vertices
	std::map<GLuint,VertexVectors>::iterator it;
	for(it=gri.getGlCoords().begin();it!=gri.getGlCoords().end();++it) {

		for(m_vertCounter=0; m_vertCounter<it->second.m_vertices.size();++m_vertCounter) {
			it->second.m_vertices[m_vertCounter].x*=m_width;
			it->second.m_vertices[m_vertCounter].y*=m_height;
		}

		glBindTexture(GL_TEXTURE_2D,it->first);
		//glBindTexture(GL_TEXTURE_2D,0);
		m_verticesCounter=0;
		glTexCoordPointer(2, GL_FLOAT, 0, &(it->second.m_textureCoords[0]));
		glVertexPointer(2, GL_FLOAT, 0, &(it->second.m_vertices[0]));
		glColorPointer(4, GL_FLOAT, 0, &(it->second.m_colors[0]));
		while(m_verticesCounter<it->second.m_vertices.size()) {
			m_nbVertices=it->second.m_vertices.size()-m_verticesCounter;
			if(m_nbVertices>m_maxVerticesNb) {
				m_nbVertices = m_maxVerticesNb;
			}
			glDrawArrays(GL_QUADS, m_verticesCounter, m_nbVertices);
			m_verticesCounter+=m_nbVertices;
		}

		for(m_vertCounter=0; m_vertCounter<it->second.m_vertices.size();++m_vertCounter) {
			it->second.m_vertices[m_vertCounter].x/=m_width;
			it->second.m_vertices[m_vertCounter].y/=m_height;
		}
	}
}

void RenderWindow::drawCell(Grille& gri)
{
	//get scale
	m_cellScale = gri.getParameterValueAt(PARAM_SIZE,m_cellCounter);
	m_texScale=1.0;
	m_currentCellWidth=m_cellWidth*m_cellScale;
	m_currentCellHeight=m_cellHeight*m_cellScale;
	if(m_cellScale>1.0) {
		m_texScale=m_cellScale;
		m_currentCellWidth=m_cellWidth;
		m_currentCellHeight=m_cellHeight;
	}


	//get Position
	switch(gri.getOrigin()) {
		case ORIGIN_UP_LEFT: {
			m_posX=m_colCounter*m_cellWidth+m_cellWidth/2.0;
			m_posY=m_rowCounter*m_cellHeight+m_cellHeight/2.0;
		}break;
		case ORIGIN_UP_RIGHT: {
			/*
			m_posX=(m_width-m_cellWidth)-m_colCounter*m_cellWidth + m_cellWidth/2.0;
			m_posY=m_rowCounter*m_cellHeight+m_cellHeight/2.0;
			*/
			m_posX=(1-m_cellWidth)-m_colCounter*m_cellWidth + m_cellWidth/2.0;
			m_posY=m_rowCounter*m_cellHeight+m_cellHeight/2.0;
		}break;
		case ORIGIN_DOWN_RIGHT: {
			/*
			m_posX=(m_width-m_cellWidth)-m_colCounter*m_cellWidth + m_cellWidth/2.0;
			m_posY=(m_height-m_cellHeight)-m_rowCounter*m_cellHeight + m_cellHeight/2.0;
			*/
			m_posX=(1-m_cellWidth)-m_colCounter*m_cellWidth + m_cellWidth/2.0;
			m_posY=(1-m_cellHeight)-m_rowCounter*m_cellHeight + m_cellHeight/2.0;
		}break;
		case ORIGIN_DOWN_LEFT: {
			m_posX=m_colCounter*m_cellWidth+m_cellWidth/2.0;
			//m_posY=((m_height-m_cellHeight)-m_rowCounter*m_cellHeight) + m_cellHeight/2.0;
			m_posY=((1-m_cellHeight)-m_rowCounter*m_cellHeight) + m_cellHeight/2.0;
		}break;
	}

	//get color parameter
	Parameter &colorParam = gri.getParameter(PARAM_COLOR);

	//compute rotation
	m_currentCellRot = (gri.getParameterValueAt(PARAM_ROTATION,m_cellCounter)/360.0)*(2.0*M_PI);
	m_currentCellDiag = sqrt(pow(m_currentCellWidth/2.0, 2.0) + pow(m_currentCellHeight/2.0, 2.0));
	m_currentCellAngle = acos((m_currentCellWidth/2.0)/(m_currentCellDiag));
	m_currentCellDisX = gri.getParameterValueAt(PARAM_DISX, m_cellCounter);
	m_currentCellDisY = gri.getParameterValueAt(PARAM_DISY, m_cellCounter);

	m_currentCellImage = gri.getImageAt(m_cellCounter);

	//FILL ARRAYS
	if(gri.getGlCoords().find(m_currentCellImage)==gri.getGlCoords().end()) {
		gri.getGlCoords()[m_currentCellImage] = VertexVectors();
	}
	

	//vertices
	gri.getGlCoords()[m_currentCellImage].m_vertices.push_back(Vertex((m_posX+m_currentCellDisX+cos(M_PI+m_currentCellAngle+m_currentCellRot)*m_currentCellDiag), m_posY+m_currentCellDisY+sin(M_PI+m_currentCellAngle+m_currentCellRot)*m_currentCellDiag)); 
	gri.getGlCoords()[m_currentCellImage].m_vertices.push_back(Vertex(m_posX+m_currentCellDisX+cos(m_currentCellRot-m_currentCellAngle)*m_currentCellDiag, m_posY+m_currentCellDisY+sin(m_currentCellRot-m_currentCellAngle)*m_currentCellDiag)); 
	gri.getGlCoords()[m_currentCellImage].m_vertices.push_back(Vertex(m_posX+m_currentCellDisX+cos(m_currentCellAngle+m_currentCellRot)*m_currentCellDiag, m_posY+m_currentCellDisY+sin(m_currentCellAngle+m_currentCellRot)*m_currentCellDiag)); 
	gri.getGlCoords()[m_currentCellImage].m_vertices.push_back(Vertex(m_posX+m_currentCellDisX+cos((M_PI-m_currentCellAngle)+m_currentCellRot)*m_currentCellDiag, m_posY+m_currentCellDisY+sin((M_PI-m_currentCellAngle)+m_currentCellRot)*m_currentCellDiag)); 

	//colors
	gri.getGlCoords()[m_currentCellImage].m_colors.push_back(VertexColor(colorParam.getColorValueAt(m_cellCounter,0)/255.0,colorParam.getColorValueAt(m_cellCounter,1)/255.0,colorParam.getColorValueAt(m_cellCounter,2)/255.0,((gri.getParameterValueAt(PARAM_TRANS,m_cellCounter)-1.0)*-1.0)*((gri.getLoopTransparency()-1.0)*-1.0))); 
	gri.getGlCoords()[m_currentCellImage].m_colors.push_back(VertexColor(colorParam.getColorValueAt(m_cellCounter,0)/255.0, colorParam.getColorValueAt(m_cellCounter,1)/255.0, colorParam.getColorValueAt(m_cellCounter,2)/255.0, ((gri.getParameterValueAt(PARAM_TRANS,m_cellCounter)-1.0)*-1.0)*((gri.getLoopTransparency()-1.0)*-1.0))); 
	gri.getGlCoords()[m_currentCellImage].m_colors.push_back(VertexColor(colorParam.getColorValueAt(m_cellCounter,0)/255.0,colorParam.getColorValueAt(m_cellCounter,1)/255.0,colorParam.getColorValueAt(m_cellCounter,2)/255.0,((gri.getParameterValueAt(PARAM_TRANS,m_cellCounter)-1.0)*-1.0)*((gri.getLoopTransparency()-1.0)*-1.0))); 
	gri.getGlCoords()[m_currentCellImage].m_colors.push_back( VertexColor(colorParam.getColorValueAt(m_cellCounter,0)/255.0, colorParam.getColorValueAt(m_cellCounter,1)/255.0, colorParam.getColorValueAt(m_cellCounter,2)/255.0, ((gri.getParameterValueAt(PARAM_TRANS,m_cellCounter)-1.0)*-1.0)*((gri.getLoopTransparency()-1.0)*-1.0))); 

	//texture coords
	gri.getGlCoords()[m_currentCellImage].m_textureCoords.push_back(VertexTexture(0.5-0.5/m_texScale, 0.5-0.5/m_texScale));
	gri.getGlCoords()[m_currentCellImage].m_textureCoords.push_back(VertexTexture(0.5+0.5/m_texScale, 0.5-0.5/m_texScale));
	gri.getGlCoords()[m_currentCellImage].m_textureCoords.push_back(VertexTexture(0.5+0.5/m_texScale, 0.5+0.5/m_texScale));
	gri.getGlCoords()[m_currentCellImage].m_textureCoords.push_back(VertexTexture(0.5-0.5/m_texScale, 0.5+0.5/m_texScale));
}


int RenderWindow::handle(int event)
{
	if(m_resizable) {
		/*
		switch(event) {
			case FL_KEYDOWN: {
				if(Fl::event_key()==32) {
					toggleFullscreen();
					return 1;
				}
			}break;
		}*/
		
		switch(event) {
			case FL_PUSH: {
				if(Fl::event_button()==FL_LEFT_MOUSE) {
					if(Fl::event_clicks()>=1) {
						toggleFullscreen();
						Fl::event_clicks(0);
						Fl::event_is_click(0);
					}
				}
				return 1;
			}break;
			default: {
				return Fl_Widget::handle(event);
			}break;
		}
	}
	return Fl_Widget::handle(event);
}

void RenderWindow::toggleFullscreen()
{
	
	m_fullscreen = ! m_fullscreen;

	if(m_fullscreen) {
		cursor(FL_CURSOR_NONE);
        fullscreen();
	}
	else {
		cursor(FL_CURSOR_DEFAULT);
        fullscreen_off(0, 0, 800, 600);
	}

}

void RenderWindow::drawGrilleOld(Grille& gri)
{
	//float width=4.0;
	//float height=3.0;
	float width=this->w();
	float height=this->h();
	float nbRows=floor(gri.getParameter(PARAM_ROWS).getFirstElementValue());
	float nbCols=floor(gri.getParameter(PARAM_COLS).getFirstElementValue());
	//cout<<"grille:"<<nbRows<<":"<<nbCols<<endl;
	float cellWidth=(width/nbCols);
	float cellHeight=(height/nbRows);
	if(gri.getHomothety()) {
		if(nbRows>nbCols) {
			cellWidth=width/nbRows;
		}
		else {
			cellHeight=height/nbCols;
		}
	}

	if(gri.getBlend()) {
		//glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	}
	else {
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		//glDisable(GL_BLEND);
	}

	//for each row and column, draw the cells
	int cellCounter=0;
	for(float r=0;r<nbRows;++r) {
		for(float c=0;c<nbCols;++c,++cellCounter) {
			drawCellOld(gri, c, r, cellCounter, cellWidth, cellHeight);
		}
	}

}

void RenderWindow::drawCellOld(Grille& gri, int c, int r, int cellCounter,float cellWidth, float cellHeight)
{
	//get scale
	float cellScale = gri.getParameterValueAt(PARAM_SIZE,cellCounter);
	float texScale=1.0;
	float currentCellWidth=cellWidth*cellScale;
	float currentCellHeight=cellHeight*cellScale;
	if(cellScale>1.0) {
		texScale=cellScale;
		currentCellWidth=cellWidth;
		currentCellHeight=cellHeight;
	}

	//get color
	Parameter &colorParam = gri.getParameter(PARAM_COLOR);
	glColor4f(colorParam.getColorValueAt(cellCounter,0)/255.0,colorParam.getColorValueAt(cellCounter,1)/255.0,colorParam.getColorValueAt(cellCounter,2)/255.0,((gri.getParameterValueAt(PARAM_TRANS,cellCounter)-1.0)*-1.0)*((gri.getLoopTransparency()-1.0)*-1.0));


	//get texture
	//if(gri.getImageAt(cellCounter)>0) {
		glBindTexture(GL_TEXTURE_2D,gri.getImageAt(cellCounter));
	//}

	//get Position
	float posX=0,posY=0;
	switch(gri.getOrigin()) {
		case ORIGIN_UP_LEFT: {
			posX=c*cellWidth+cellWidth/2.0;
			posY=r*cellHeight+cellHeight/2.0;
		}break;
		case ORIGIN_UP_RIGHT: {
			posX=(float(this->w()-cellWidth)-c*cellWidth) + cellWidth/2.0;
			posY=r*cellHeight+cellHeight/2.0;
		}break;
		case ORIGIN_DOWN_RIGHT: {
			posX=(float(this->w()-cellWidth)-c*cellWidth) + cellWidth/2.0;
			posY=(float(this->h()-cellHeight)-r*cellHeight) + cellHeight/2.0;
		}break;
		case ORIGIN_DOWN_LEFT: {
			posX=c*cellWidth+cellWidth/2.0;
			posY=(float(this->h()-cellHeight)-r*cellHeight) + cellHeight/2.0;
		}break;
	}

	glPushMatrix();
		//rotate and translate
		glTranslatef(posX,posY,0.0f);
		glRotatef(gri.getParameterValueAt(PARAM_ROTATION,cellCounter),0.0f,0.0f,1.0f);
		
		glScalef(currentCellWidth,currentCellHeight,cellScale);
		//draw the quad with texture
		glBegin(GL_QUADS);
			glTexCoord2f (0.5-0.5/texScale, 0.5-0.5/texScale);glVertex2f (-0.5, -0.5);
			glTexCoord2f (0.5+0.5/texScale, 0.5-0.5/texScale);glVertex2f (0.5, -0.5);
			glTexCoord2f (0.5+0.5/texScale, 0.5+0.5/texScale);glVertex2f (0.5, 0.5);
			glTexCoord2f (0.5-0.5/texScale, 0.5+0.5/texScale);glVertex2f (-0.5, 0.5);
		glEnd ();
	glPopMatrix();

}
