/***************************************************************************
 *            RenderWindow.h
 *
 *  Mon Sep  4 09:56:39 2006
 *  Copyright  2006  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/gl.h>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Toggle_Button.H>
#include <FL/Fl_Slider.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Counter.H>
#include <iostream>
#include <string>
#include <vector>
#include <map>

#include <cmath>
 
#ifndef _RENDERWINDOW_H
#define _RENDERWINDOW_H

#include "../base/Grille.hpp"

enum MOUSE_BUTTON {NO_BUTTON, LEFT_BUTTON, RIGHT_BUTTON};
enum RENDER_WINDOW_TYPE { RENDER_WINDOW, RENDER_MONITOR, RENDER_GRILLE, RENDER_LOOP};

class RenderWindow : public Fl_Gl_Window 
{
  public:
		RenderWindow(int , int ,const char*, RENDER_WINDOW_TYPE type=RENDER_WINDOW, int id=0);
		RenderWindow(int , int , int, int, const char*, RENDER_WINDOW_TYPE type=RENDER_GRILLE, int id=0);
		~RenderWindow();
		void setTitle(std::string);
		int handle(int);

		inline void generateTexture(GLuint *id){make_current();glGenTextures(1,id);}
		inline void deleteTexture(GLuint *id){make_current();glDeleteTextures(1,id);}

		void draw();
		void drawGrilleOld(Grille&);
		void drawCellOld(Grille&, int, int, int, float, float);
		
		void drawGrille(Grille&);
		void drawCell(Grille&);

		inline void setResizable(bool res) {m_resizable=res;}

		void toggleFullscreen();	

		inline void setGrille(int id) {m_grilleID=id;}

  private:
		bool m_isFullScreen;
		MOUSE_BUTTON m_mouseButtonPressed;
		int m_mouseX;
		int m_mouseY;
		bool m_resizable;
		bool m_fullscreen;

		int m_grilleID;
		RENDER_WINDOW_TYPE m_type;


		std::map<GLuint, VertexVectors> m_texturesVerticesMap;

		GLuint m_vertexVboID; 
		GLuint m_colorsVboID; 
		GLuint m_texturesVboID; 

		int m_maxVerticesNb;
		int m_verticesCounter;
		int m_nbVertices;
		
		int m_cellCounter;
		float m_rowCounter;
		float m_colCounter;

		float m_width;
		float m_height;
		float m_nbRows;
		float m_nbCols;
		float m_cellWidth;
		float m_cellHeight;

		float m_texScale;
		float m_cellScale;
		float m_currentCellWidth;
		float m_currentCellHeight;
		float m_currentCellRot;
		float m_currentCellDiag;
		float m_currentCellAngle;
		GLuint m_currentCellImage;
		float m_posX;
		float m_posY;
		float m_currentCellDisX;
		float m_currentCellDisY;

		int m_vertCounter;

		bool m_initialized;
};

#endif /* _EXECWINDOW_H */
